@@ -26,6 +26,7 @@ namespace Mono.Debugger.Soft
         public Mono.Debugger.Soft.AssemblyMirror Corlib { get { throw null; } }
         public string FriendlyName { get { throw null; } }
         public Mono.Debugger.Soft.ObjectMirror CreateBoxedValue(Mono.Debugger.Soft.Value value) { throw null; }
+        public Mono.Debugger.Soft.ArrayMirror CreateByteArray(byte[] bytes) { throw null; }
         public Mono.Debugger.Soft.StringMirror CreateString(string s) { throw null; }
         public Mono.Debugger.Soft.AssemblyMirror[] GetAssemblies() { throw null; }
         public Mono.Debugger.Soft.TypeMirror GetCorrespondingType(System.Type t) { throw null; }
@@ -113,6 +114,7 @@ namespace Mono.Debugger.Soft
         protected abstract int TransportReceive(byte[] buf, int buf_offset, int len);
         protected abstract int TransportSend(byte[] buf, int buf_offset, int len);
         protected abstract void TransportSetTimeouts(int send_timeout, int receive_timeout);
+        protected abstract void TransportShutdown();
         public long[] Type_GetMethodsByNameFlags(long id, string name, int flags, bool ignoreCase) { throw null; }
     }
     public partial class CrashEvent : Mono.Debugger.Soft.Event
@@ -253,6 +255,7 @@ namespace Mono.Debugger.Soft
         public System.Reflection.FieldAttributes Attributes { get { throw null; } }
         public Mono.Debugger.Soft.TypeMirror DeclaringType { get { throw null; } }
         public Mono.Debugger.Soft.TypeMirror FieldType { get { throw null; } }
+        public int FixedSize { get { throw null; } }
         public string FullName { get { throw null; } }
         public bool IsAssembly { get { throw null; } }
         public bool IsFamily { get { throw null; } }
@@ -834,6 +837,7 @@ namespace Mono.Debugger.Soft
         public Mono.Debugger.Soft.EnumMirror CreateEnumMirror(Mono.Debugger.Soft.TypeMirror type, Mono.Debugger.Soft.PrimitiveValue value) { throw null; }
         public Mono.Debugger.Soft.ExceptionEventRequest CreateExceptionRequest(Mono.Debugger.Soft.TypeMirror exc_type) { throw null; }
         public Mono.Debugger.Soft.ExceptionEventRequest CreateExceptionRequest(Mono.Debugger.Soft.TypeMirror exc_type, bool caught, bool uncaught) { throw null; }
+        public Mono.Debugger.Soft.ExceptionEventRequest CreateExceptionRequest(Mono.Debugger.Soft.TypeMirror exc_type, bool caught, bool uncaught, bool everything_else) { throw null; }
         public Mono.Debugger.Soft.MethodEntryEventRequest CreateMethodEntryRequest() { throw null; }
         public Mono.Debugger.Soft.MethodExitEventRequest CreateMethodExitRequest() { throw null; }
         public Mono.Debugger.Soft.StepEventRequest CreateStepRequest(Mono.Debugger.Soft.ThreadMirror thread) { throw null; }
