@@ -95,6 +95,8 @@ namespace ILLink.Tasks
 
 
 		/// ToolTask implementation
+		
+		protected override MessageImportance StandardErrorLoggingImportance => MessageImportance.High;
 
 		protected override string ToolName => Path.GetFileName (DotNetPath);
 
@@ -181,9 +183,7 @@ namespace ILLink.Tasks
 				args.Append ("-out ").AppendLine (Quote (OutputDirectory.ItemSpec));
 
 			if (ClearInitLocals) {
-				args.Append ("-s ");
-				// Version of ILLink.CustomSteps is passed as a workaround for msbuild issue #3016
-				args.AppendLine ("ILLink.CustomSteps.ClearInitLocalsStep,ILLink.CustomSteps,Version=0.0.0.0:OutputStep");
+				args.AppendLine ("--enable-opt clearinitlocals");
 				if ((ClearInitLocalsAssemblies != null) && (ClearInitLocalsAssemblies.Length > 0)) {
 					args.Append ("-m ClearInitLocalsAssemblies ");
 					args.AppendLine (ClearInitLocalsAssemblies);
