@@ -35,11 +35,11 @@ using Mono.Cecil.Cil;
 
 namespace Mono.Linker.Steps {
 
-	public class SweepStep : BaseStep {
-
+	public class SweepStep : BaseStep
+	{
 		AssemblyDefinition [] assemblies;
-		HashSet<AssemblyDefinition> resolvedTypeReferences;
 		readonly bool sweepSymbols;
+		readonly HashSet<AssemblyDefinition> BypassNGenToSave = new HashSet<AssemblyDefinition> ();
 
 		public SweepStep (bool sweepSymbols = true)
 		{
@@ -49,61 +49,78 @@ namespace Mono.Linker.Steps {
 		protected override void Process ()
 		{
 			assemblies = Context.Annotations.GetAssemblies ().ToArray ();
+
+			foreach (var assembly in assemblies) {
+				RemoveUnusedAssembly (assembly);
+			}
+
 			foreach (var assembly in assemblies) {
 				ProcessAssemblyAction (assembly);
-				if ((Annotations.GetAction (assembly) == AssemblyAction.Copy) &&
-					!Context.KeepTypeForwarderOnlyAssemblies) {
-						// Copy assemblies can still contain Type references with
-						// type forwarders from Delete assemblies
-						// thus try to resolve all the type references and see
-						// if some changed the scope. if yes change the action to Save
-						if (ResolveAllTypeReferences (assembly))
-							Annotations.SetAction (assembly, AssemblyAction.Save);
-				}
+			}
+		}
 
-				AssemblyAction currentAction = Annotations.GetAction (assembly);
+		void RemoveUnusedAssembly (AssemblyDefinition assembly)
+		{
+			switch (Annotations.GetAction (assembly)) {
+			case AssemblyAction.AddBypassNGenUsed:
+			case AssemblyAction.CopyUsed:
+			case AssemblyAction.Link:
+				if (!IsUsedAssembly (assembly))
+					RemoveAssembly (assembly);
 
-				if ((currentAction == AssemblyAction.Link) || (currentAction == AssemblyAction.Save)) {
-					// if we save (only or by linking) then unmarked exports (e.g. forwarders) must be cleaned
-					// or they can point to nothing which will break later (e.g. when re-loading for stripping IL)
-					// reference: https://bugzilla.xamarin.com/show_bug.cgi?id=36577
-					if (assembly.MainModule.HasExportedTypes)
-						SweepCollectionNonAttributable (assembly.MainModule.ExportedTypes);
-				}
+				break;
 			}
 		}
 
 		protected void ProcessAssemblyAction (AssemblyDefinition assembly)
 		{
 			switch (Annotations.GetAction (assembly)) {
-				case AssemblyAction.Link:
-					if (!IsMarkedAssembly (assembly)) {
-						RemoveAssembly (assembly);
-						return;
-					}
-					break;
+			case AssemblyAction.AddBypassNGenUsed:
+				Annotations.SetAction (assembly, AssemblyAction.AddBypassNGen);
+				goto case AssemblyAction.AddBypassNGen;
 
-				case AssemblyAction.AddBypassNGenUsed:
-					if (!IsMarkedAssembly (assembly)) {
-						RemoveAssembly (assembly);
-					} else {
-						Annotations.SetAction (assembly, AssemblyAction.AddBypassNGen);
-					}
-					return;
+			case AssemblyAction.AddBypassNGen:
+				// FIXME: AddBypassNGen is just wrong, it should not be action as we need to
+				// turn it to Action.Save here to e.g. correctly update debug symbols
+				if (!Context.KeepTypeForwarderOnlyAssemblies || BypassNGenToSave.Contains (assembly)) {
+					goto case AssemblyAction.Save;
+				}
 
-				case AssemblyAction.CopyUsed:
-					if (!IsMarkedAssembly (assembly)) {
-						RemoveAssembly (assembly);
-					} else {
-						Annotations.SetAction (assembly, AssemblyAction.Copy);
-					}
-					return;
+				break;
 
-				default:
-					return;
+			case AssemblyAction.CopyUsed:
+				Annotations.SetAction (assembly, AssemblyAction.Copy);
+				goto case AssemblyAction.Copy;
+
+			case AssemblyAction.Copy:
+				//
+				// Facade assemblies can have unused forwarders pointing to
+				// removed type (when facades are kept)
+				//
+				//		main.exe -> facade.dll -> lib.dll
+				//		link     |  copy       |  link
+				//
+				// when main.exe has unused reference to type in lib.dll
+				//
+				if (SweepTypeForwarders (assembly))
+					Annotations.SetAction (assembly, AssemblyAction.Save);
+
+				break;
+
+			case AssemblyAction.Link:
+				SweepAssembly (assembly);
+				break;
+
+			case AssemblyAction.Save:
+				//
+				// Save means we need to rewrite the assembly due to removed assembly
+				// reference. We do any additional removed assembly reference clean up here
+				//
+				UpdateForwardedTypesScope (assembly);
+				UpdateCustomAttributesTypesScopes (assembly);
+				SweepTypeForwarders (assembly);
+				break;
 			}
-			
-			SweepAssembly (assembly);
 		}
 
 		protected virtual void SweepAssembly (AssemblyDefinition assembly)
@@ -132,6 +149,21 @@ namespace Mono.Linker.Steps {
 
 			foreach (var module in assembly.Modules)
 				SweepCustomAttributes (module);
+
+			SweepTypeForwarders (assembly);
+
+			UpdateForwardedTypesScope (assembly);
+		}
+
+		bool IsUsedAssembly (AssemblyDefinition assembly)
+		{
+			if (IsMarkedAssembly (assembly))
+				return true;
+
+			if (assembly.MainModule.HasExportedTypes && Context.KeepTypeForwarderOnlyAssemblies)
+				return true;
+
+			return false;
 		}
 
 		bool IsMarkedAssembly (AssemblyDefinition assembly)
@@ -143,7 +175,15 @@ namespace Mono.Linker.Steps {
 		{
 			Annotations.SetAction (assembly, AssemblyAction.Delete);
 
-			SweepReferences (assembly);
+			foreach (var a in assemblies) {
+				switch (Annotations.GetAction (a)) {
+				case AssemblyAction.Skip:
+				case AssemblyAction.Delete:
+					continue;
+				}
+
+				SweepReferences (a, assembly);
+			}
 		}
 
 		void SweepResources (AssemblyDefinition assembly)
@@ -163,115 +203,242 @@ namespace Mono.Linker.Steps {
 			}
 		}
 
-		void SweepReferences (AssemblyDefinition target)
+		void SweepReferences (AssemblyDefinition assembly, AssemblyDefinition referenceToRemove)
 		{
-			foreach (var assembly in assemblies)
-				SweepReferences (assembly, target);
-		}
-
-		void SweepReferences (AssemblyDefinition assembly, AssemblyDefinition target)
-		{
-			if (assembly == target)
+			if (assembly == referenceToRemove)
 				return;
 
+			bool reference_removed = false;
+
 			var references = assembly.MainModule.AssemblyReferences;
 			for (int i = 0; i < references.Count; i++) {
 				var reference = references [i];
-				AssemblyDefinition r = Context.Resolver.Resolve (reference);
-				if (r == null)
-					continue;
-				if (!AreSameReference (r.Name, target.Name))
+
+				AssemblyDefinition ad = Context.Resolver.Resolve (reference);
+				if (ad == null || !AreSameReference (ad.Name, referenceToRemove.Name))
 					continue;
 
 				ReferenceRemoved (assembly, reference);
-				// removal from `references` requires an adjustment to `i`
 				references.RemoveAt (i--);
-				// Removing the reference does not mean it will be saved back to disk!
-				// That depends on the AssemblyAction set for the `assembly`
+				reference_removed = true;
+			}
+
+			if (reference_removed) {
 				switch (Annotations.GetAction (assembly)) {
-				case AssemblyAction.Copy:
-					// We need to save the assembly if a reference was removed, otherwise we can end up
-					// with an assembly that references an assembly that no longer exists
-					Annotations.SetAction (assembly, AssemblyAction.Save);
-					// Copy means even if "unlinked" we still want that assembly to be saved back 
-					// to disk (OutputStep) without the (removed) reference
-					if (!Context.KeepTypeForwarderOnlyAssemblies) {
-						ResolveAllTypeReferences (assembly);
-					}
-					break;
-
 				case AssemblyAction.CopyUsed:
-					if (IsMarkedAssembly (assembly) && !Context.KeepTypeForwarderOnlyAssemblies) {
-						Annotations.SetAction (assembly, AssemblyAction.Save);
-						ResolveAllTypeReferences (assembly);
+					if (IsUsedAssembly (assembly)) {
+						goto case AssemblyAction.Copy;
 					}
 					break;
 
-				case AssemblyAction.Save:
-				case AssemblyAction.Link:
-				case AssemblyAction.AddBypassNGen:
+				case AssemblyAction.Copy:
+					//
+					// Assembly has a reference to another assembly which has been fully removed. This can
+					// happen when for example the reference assembly is 'copy-used' and it's not needed.
+					//
+					// or
+					//
+					// Assembly can contain type references with
+					// type forwarders to deleted assembly (facade) when
+					// facade assemblies are not kept. For that reason we need to
+					// rewrite the copy to save to update the scopes not to point
+					// forwardning assembly (facade).
+					//
+					//		foo.dll -> facade.dll    -> lib.dll
+					//		copy    |  copy (delete) |  link
+					//
+					Annotations.SetAction (assembly, AssemblyAction.Save);
+					break;
+
 				case AssemblyAction.AddBypassNGenUsed:
-					if (!Context.KeepTypeForwarderOnlyAssemblies) {
-						ResolveAllTypeReferences (assembly);
+					if (IsUsedAssembly (assembly)) {
+						Annotations.SetAction (assembly, AssemblyAction.AddBypassNGen);
+						goto case AssemblyAction.AddBypassNGen;
 					}
 					break;
+
+				case AssemblyAction.AddBypassNGen:
+					BypassNGenToSave.Add (assembly);
+					break;
 				}
 			}
 		}
 
-		bool ResolveAllTypeReferences (AssemblyDefinition assembly)
+		bool SweepTypeForwarders (AssemblyDefinition assembly)
 		{
-			if (resolvedTypeReferences == null)
-				resolvedTypeReferences = new HashSet<AssemblyDefinition> ();
-			if (resolvedTypeReferences.Contains (assembly))
-				return false;
-			resolvedTypeReferences.Add (assembly);
+			if (assembly.MainModule.HasExportedTypes) {
+				return SweepCollectionMetadata (assembly.MainModule.ExportedTypes);
+			}
 
-			var hash = new Dictionary<TypeReference,IMetadataScope> ();
-			bool changes = false;
+			return false;
+		}
+
+		void UpdateForwardedTypesScope (AssemblyDefinition assembly)
+		{
+			var changed_types = new Dictionary<TypeReference, IMetadataScope> ();
 
 			foreach (TypeReference tr in assembly.MainModule.GetTypeReferences ()) {
-				if (hash.ContainsKey (tr))
-					continue;
 				if (tr.IsWindowsRuntimeProjection)
 					continue;
+
 				var td = tr.Resolve ();
-				IMetadataScope scope = tr.Scope;
 				// at this stage reference might include things that can't be resolved
 				// and if it is (resolved) it needs to be kept only if marked (#16213)
-				if ((td != null) && Annotations.IsMarked (td)) {
-					scope = assembly.MainModule.ImportReference (td).Scope;
-					if (tr.Scope != scope)
-						changes = true;
-					hash.Add (tr, scope);
-				}
-			}
-			if (assembly.MainModule.HasExportedTypes) {
-				foreach (var et in assembly.MainModule.ExportedTypes) {
-					var td = et.Resolve ();
-					IMetadataScope scope = et.Scope;
-					if ((td != null) && Annotations.IsMarked (td)) {
-						scope = assembly.MainModule.ImportReference (td).Scope;
-						et.Scope = scope;
-					}
-				}
+				if (td == null || !Annotations.IsMarked (td))
+					continue;
+
+				IMetadataScope scope = assembly.MainModule.ImportReference (td).Scope;
+				if (tr.Scope != scope)
+					changed_types.Add (tr, scope);
 			}
 
-			// Resolve everything first before updating scopes.
+			//
+			// Resolved everything first before updating scopes.
 			// If we set the scope to null, then calling Resolve() on any of its
 			// nested types would crash.
-
-			foreach (var e in hash) {
+			//
+			foreach (var e in changed_types) {
 				e.Key.Scope = e.Value;
 			}
 
-			return changes;
+			if (assembly.MainModule.HasExportedTypes) {
+				foreach (var et in assembly.MainModule.ExportedTypes) {
+					var td = et.Resolve ();
+					if (td == null)
+						continue;
+
+					et.Scope = assembly.MainModule.ImportReference (td).Scope;
+				}
+			}
+		}
+
+		static void UpdateCustomAttributesTypesScopes (AssemblyDefinition assembly)
+		{
+			UpdateCustomAttributesTypesScopes ((ICustomAttributeProvider) assembly);
+
+			foreach (var module in assembly.Modules)
+				UpdateCustomAttributesTypesScopes (module);
+
+			foreach (var type in assembly.MainModule.Types)
+				UpdateCustomAttributesTypesScopes (type);
+		}
+
+		static void UpdateCustomAttributesTypesScopes (TypeDefinition typeDefinition)
+		{
+			UpdateCustomAttributesTypesScopes ((ICustomAttributeProvider)typeDefinition);
+
+			if (typeDefinition.HasEvents)
+				UpdateCustomAttributesTypesScopes (typeDefinition.Events);
+
+			if (typeDefinition.HasFields)
+				UpdateCustomAttributesTypesScopes (typeDefinition.Fields);
+
+			if (typeDefinition.HasMethods)
+				UpdateCustomAttributesTypesScopes (typeDefinition.Methods);
+
+			if (typeDefinition.HasProperties)
+				UpdateCustomAttributesTypesScopes (typeDefinition.Properties);
+
+			if (typeDefinition.HasGenericParameters)
+				UpdateCustomAttributesTypesScopes (typeDefinition.GenericParameters);
+
+			if (typeDefinition.HasNestedTypes) {
+				foreach (var nestedType in typeDefinition.NestedTypes) {
+					UpdateCustomAttributesTypesScopes (nestedType);
+				}
+			}
+		}
+
+		static void UpdateCustomAttributesTypesScopes<T> (Collection<T> providers) where T : ICustomAttributeProvider
+		{
+			foreach (var provider in providers)
+				UpdateCustomAttributesTypesScopes (provider);
+		}
+
+		static void UpdateCustomAttributesTypesScopes (Collection<GenericParameter> genericParameters)
+		{
+			foreach (var gp in genericParameters) {
+				UpdateCustomAttributesTypesScopes (gp);
+
+				if (gp.HasConstraints)
+					UpdateCustomAttributesTypesScopes (gp.Constraints);
+			}
+		}
+
+		static void UpdateCustomAttributesTypesScopes (ICustomAttributeProvider customAttributeProvider)
+		{
+			if (!customAttributeProvider.HasCustomAttributes)
+				return;
+
+			foreach (var ca in customAttributeProvider.CustomAttributes)
+				UpdateForwardedTypesScope (ca);
+		}
+
+		static void UpdateForwardedTypesScope (CustomAttribute attribute)
+		{
+			AssemblyDefinition assembly = attribute.Constructor.Module.Assembly;
+
+			if (attribute.HasConstructorArguments) {
+				foreach (var ca in attribute.ConstructorArguments)
+					UpdateForwardedTypesScope (ca, assembly);
+			}
+
+			if (attribute.HasFields) {
+				foreach (var field in attribute.Fields)
+					UpdateForwardedTypesScope (field.Argument, assembly);
+			}
+
+			if (attribute.HasProperties) {
+				foreach (var property in attribute.Properties)
+					UpdateForwardedTypesScope (property.Argument, assembly);
+			}
+		}
+
+		static void UpdateForwardedTypesScope (CustomAttributeArgument attributeArgument, AssemblyDefinition assembly)
+		{
+			UpdateTypeScope (attributeArgument.Type, assembly);
+
+			switch (attributeArgument.Value) {
+			case TypeReference tr:
+				UpdateTypeScope (tr, assembly);
+				break;
+			case CustomAttributeArgument caa:
+				UpdateForwardedTypesScope (caa, assembly);
+				break;
+			case CustomAttributeArgument[] array:
+				foreach (var item in array)
+					UpdateForwardedTypesScope (item, assembly);
+				break;
+			}
+		}
+
+		static void UpdateTypeScope (TypeReference type, AssemblyDefinition assembly)
+		{
+			if (type is GenericInstanceType git && git.HasGenericArguments) {
+				UpdateTypeScope (git.ElementType, assembly);
+				foreach (var ga in git.GenericArguments)
+					UpdateTypeScope (ga, assembly);
+				return;
+			}
+
+			if (type is ArrayType at) {
+				UpdateTypeScope (at.ElementType, assembly);
+				return;
+			}
+
+			TypeDefinition td = type.Resolve ();
+			if (td == null)
+				return;
+
+			IMetadataScope scope = assembly.MainModule.ImportReference (td).Scope;
+			if (type.Scope != scope)
+				type.Scope = td.Scope;
 		}
 
 		protected virtual void SweepType (TypeDefinition type)
 		{
 			if (type.HasFields)
-				SweepCollection (type.Fields);
+				SweepCollectionWithCustomAttributes (type.Fields);
 
 			if (type.HasMethods)
 				SweepMethods (type.Methods);
@@ -285,6 +452,9 @@ namespace Mono.Linker.Steps {
 			if (type.HasCustomAttributes)
 				SweepCustomAttributes (type);
 
+			if (type.HasGenericParameters)
+				SweepGenericParameters (type.GenericParameters);
+
 			if (type.HasProperties)
 				SweepCustomAttributeCollection (type.Properties);
 
@@ -312,13 +482,25 @@ namespace Mono.Linker.Steps {
 		{
 			for (int i = type.Interfaces.Count - 1; i >= 0; i--) {
 				var iface = type.Interfaces [i];
-				if (Annotations.IsMarked (iface))
+				if (Annotations.IsMarked (iface)) {
+					SweepCustomAttributes (iface);
 					continue;
+				}
 				InterfaceRemoved (type, iface);
 				type.Interfaces.RemoveAt (i);
 			}
 		}
 
+		protected void SweepGenericParameters (Collection<GenericParameter> genericParameters)
+		{
+			foreach (var gp in genericParameters) {
+				SweepCustomAttributes (gp);
+
+				if (gp.HasConstraints)
+					SweepCustomAttributeCollection (gp.Constraints);
+			}
+		}
+
 		protected void SweepCustomAttributes (TypeDefinition type)
 		{
 			var removed = SweepCustomAttributes (type as ICustomAttributeProvider);
@@ -375,7 +557,9 @@ namespace Mono.Linker.Steps {
 
 			for (int i = provider.CustomAttributes.Count - 1; i >= 0; i--) {
 				var attribute = provider.CustomAttributes [i];
-				if (!Annotations.IsMarked (attribute)) {
+				if (Annotations.IsMarked (attribute)) {
+					UpdateForwardedTypesScope (attribute);
+				} else {
 					CustomAttributeUsageRemoved (provider, attribute);
 					removed.Add (provider.CustomAttributes [i]);
 					provider.CustomAttributes.RemoveAt (i);
@@ -393,18 +577,21 @@ namespace Mono.Linker.Steps {
 
 		protected virtual void SweepMethods (Collection<MethodDefinition> methods)
 		{
-			SweepCollection (methods);
+			SweepCollectionWithCustomAttributes (methods);
 			if (sweepSymbols)
 				SweepDebugInfo (methods);
 
 			foreach (var method in methods) {
+				if (method.HasGenericParameters)
+					SweepGenericParameters (method.GenericParameters);
+
+				SweepCustomAttributes (method.MethodReturnType);
+
 				if (!method.HasParameters)
 					continue;
 
 				foreach (var parameter in method.Parameters)
 					SweepCustomAttributes (parameter);
-
-				SweepCustomAttributes (method.MethodReturnType);
 			}
 		}
 
@@ -453,7 +640,7 @@ namespace Mono.Linker.Steps {
 			}
 		}
 
-		protected void SweepCollection (IList<MethodDefinition> list)
+		protected void SweepCollectionWithCustomAttributes<T> (IList<T> list) where T : ICustomAttributeProvider
 		{
 			for (int i = 0; i < list.Count; i++)
 				if (ShouldRemove (list [i])) {
@@ -464,24 +651,19 @@ namespace Mono.Linker.Steps {
 				}
 		}
 
-		protected void SweepCollection<T> (IList<T> list) where T : ICustomAttributeProvider
+		protected bool SweepCollectionMetadata<T> (IList<T> list) where T : IMetadataTokenProvider
 		{
-			for (int i = 0; i < list.Count; i++)
-				if (ShouldRemove (list [i])) {
-					ElementRemoved (list [i]);
-					list.RemoveAt (i--);
-				} else {
-					SweepCustomAttributes (list [i]);
-				}
-		}
+			bool removed = false;
 
-		protected void SweepCollectionNonAttributable<T> (IList<T> list) where T : IMetadataTokenProvider
-		{
-			for (int i = 0; i < list.Count; i++)
+			for (int i = 0; i < list.Count; i++) {
 				if (ShouldRemove (list [i])) {
 					ElementRemoved (list [i]);
 					list.RemoveAt (i--);
+					removed = true;
 				}
+			}
+
+			return removed;
 		}
 
 		protected virtual bool ShouldRemove<T> (T element) where T : IMetadataTokenProvider
