@@ -113,6 +113,8 @@ namespace Mono.Linker {
 
 		public bool KeepUsedAttributeTypesOnly { get; set; }
 
+		public bool KeepDependencyAttributes { get; set; }
+
 		public bool StripResources { get; set; }
 
 		public System.Collections.IDictionary Actions {
@@ -188,6 +190,7 @@ namespace Mono.Linker {
 
 			// See https://github.com/mono/linker/issues/612
 			DisabledOptimizations |= CodeOptimizations.UnreachableBodies;
+			DisabledOptimizations |= CodeOptimizations.ClearInitLocals;
 		}
 
 		public TypeDefinition GetType (string fullName)
@@ -284,6 +287,8 @@ namespace Mono.Linker {
 		public virtual ICollection<AssemblyDefinition> ResolveReferences (AssemblyDefinition assembly)
 		{
 			List<AssemblyDefinition> references = new List<AssemblyDefinition> ();
+			if (assembly == null)
+				return references;
 			foreach (AssemblyNameReference reference in assembly.MainModule.AssemblyReferences) {
 				AssemblyDefinition definition = Resolve (reference);
 				if (definition != null)
@@ -420,6 +425,11 @@ namespace Mono.Linker {
 		/// <summary>
 		/// Option to disable delaying marking of instance methods until an instance of that type could exist
 		/// </summary>
-		UnreachableBodies = 1 << 2
+		UnreachableBodies = 1 << 2,
+
+		/// <summary>
+		/// Option to clear the initlocals flag on methods
+		/// </summary>
+		ClearInitLocals = 1 << 3
 	}
 }
