@@ -184,6 +184,15 @@ namespace System.Net
 				}, () => Operation.Aborted, cancellationToken);
 		}
 
+		protected override bool TryReadFromBufferedContent (byte[] buffer, int offset, int count, out int result)
+		{
+			if (bufferedEntireContent && innerStream is BufferedReadStream bufferedStream)
+				return bufferedStream.TryReadFromBuffer (buffer, offset, count, out result);
+
+			result = 0;
+			return false;
+		}
+
 		bool CheckAuthHeader (string headerName)
 		{
 			var authHeader = Headers[headerName];
@@ -372,6 +381,7 @@ namespace System.Net
 				var buffer = await ReadAllAsyncInner (cancellationToken).ConfigureAwait (false);
 				var bos = new BufferOffsetSize (buffer, 0, buffer.Length, false);
 				innerStream = new BufferedReadStream (Operation, null, bos);
+				bufferedEntireContent = true;
 
 				nextReadCalled = true;
 				completion.TrySetCompleted ();
