@@ -137,15 +137,28 @@ dump_table_assemblyref (MonoImage *m)
 			 cols [MONO_ASSEMBLYREF_REV_NUMBER],
 			 mono_metadata_string_heap (m, cols [MONO_ASSEMBLYREF_NAME]));
 		fprintf (output, "\tFlags=0x%08x\n", cols [MONO_ASSEMBLYREF_FLAGS]);
-		ptr = mono_metadata_blob_heap (m, cols [MONO_ASSEMBLYREF_PUBLIC_KEY]);
-		len = mono_metadata_decode_value (ptr, &ptr);
+		ptr = mono_metadata_blob_heap_null_ok (m, cols [MONO_ASSEMBLYREF_PUBLIC_KEY]);
+		if (ptr)
+			len = mono_metadata_decode_value (ptr, &ptr);
+		else
+			len = 0;
 		if (len > 0){
 			fprintf (output, "\tPublic Key:");
 			hex_dump (ptr, 0, len);
 			fprintf (output, "\n");
 		} else
 			fprintf (output, "\tZero sized public key\n");
-		
+		ptr = mono_metadata_blob_heap_null_ok (m, cols [MONO_ASSEMBLYREF_HASH_VALUE]);
+		if (ptr)
+			len = mono_metadata_decode_value (ptr, &ptr);
+		else
+			len = 0;
+		if (len > 0) {
+			fprintf (output, "\tHash:");
+			hex_dump (ptr, 0, len);
+			fprintf (output, "\n");
+		} else
+			fprintf (output, "\tZero sized hash value\n");
 	}
 	fprintf (output, "\n");
 }
@@ -247,13 +260,13 @@ dump_table_memberref (MonoImage *m)
 		case 0:
 			ks = "TypeDef";
 			xx = get_typedef (m, idx);
-			x = g_strconcat (xx, ".", mono_metadata_string_heap (m, cols [MONO_MEMBERREF_NAME]), NULL);
+			x = g_strconcat (xx, ".", mono_metadata_string_heap (m, cols [MONO_MEMBERREF_NAME]), (const char*)NULL);
 			g_free (xx);
 			break;
 		case 1:
 			ks = "TypeRef";
 			xx = get_typeref (m, idx);
-			x = g_strconcat (xx, ".", mono_metadata_string_heap (m, cols [MONO_MEMBERREF_NAME]), NULL);
+			x = g_strconcat (xx, ".", mono_metadata_string_heap (m, cols [MONO_MEMBERREF_NAME]), (const char*)NULL);
 			g_free (xx);
 			break;
 		case 2:
@@ -265,7 +278,7 @@ dump_table_memberref (MonoImage *m)
 		case 4:
 			ks = "TypeSpec";
 			xx = get_typespec (m, idx, FALSE, NULL);
-			x = g_strconcat (xx, ".", mono_metadata_string_heap (m, cols [MONO_MEMBERREF_NAME]), NULL);
+			x = g_strconcat (xx, ".", mono_metadata_string_heap (m, cols [MONO_MEMBERREF_NAME]), (const char*)NULL);
 			g_free (xx);
 			break;
 		default:
@@ -581,26 +594,26 @@ dump_table_method (MonoImage *m)
 			type_container = mono_metadata_load_generic_params (m, MONO_TOKEN_TYPE_DEF | (current_type - 1), NULL, NULL);
 			if (type_container) {
 				mono_metadata_load_generic_param_constraints_checked (m, MONO_TOKEN_TYPE_DEF | (current_type - 1), type_container, error);
-				g_assert (mono_error_ok (error)); /*FIXME don't swallow the error message*/
+				g_assert (is_ok (error)); /*FIXME don't swallow the error message*/
 			}
 		}
 
 		method_container = mono_metadata_load_generic_params (m, MONO_TOKEN_METHOD_DEF | i, type_container, NULL);
 		if (method_container) {
 			mono_metadata_load_generic_param_constraints_checked (m, MONO_TOKEN_METHOD_DEF | i, method_container, error);
-			g_assert (mono_error_ok (error)); /*FIXME don't swallow the error message*/
+			g_assert (is_ok (error)); /*FIXME don't swallow the error message*/
 		}
 		mono_metadata_decode_table_row (m, MONO_TABLE_METHOD, i - 1, cols, MONO_METHOD_SIZE);
 		sigblob = mono_metadata_blob_heap (m, cols [MONO_METHOD_SIGNATURE]);
 		mono_metadata_decode_blob_size (sigblob, &sigblob);
 		method = mono_metadata_parse_method_signature_full (m, method_container ? method_container : type_container, i, sigblob, &sigblob, error);
-		if (!mono_error_ok (error)) {
+		if (!is_ok (error)) {
 			fprintf (output,"%d: failed to parse due to %s\n", i, mono_error_get_message (error));
 			mono_error_cleanup (error);
 			continue;
 		}
 
-		g_assert (mono_error_ok (error)); /*FIXME don't swallow the error message*/
+		g_assert (is_ok (error)); /*FIXME don't swallow the error message*/
 		sig = dis_stringify_method_signature (m, method, i, method_container ? method_container : type_container, FALSE);
                 impl_flags = get_method_impl_flags (cols [MONO_METHOD_IMPLFLAGS]);
 		fprintf (output, "%d: %s (param: %d impl_flags: %s)\n", i, sig, cols [MONO_METHOD_PARAMLIST], impl_flags);
@@ -861,9 +874,6 @@ handle_enum:
 			p += 4;
 			break;
 		case MONO_TYPE_U8:
-			g_string_append_printf (res, "%lld", (long long)read64 (p));
-			p += 8;
-			break;
 		case MONO_TYPE_I8:
 			g_string_append_printf (res, "%lld", (long long)read64 (p));
 			p += 8;
