@@ -900,13 +900,13 @@ mono_gc_wbarrier_set_arrayref_internal (MonoArray *arr, gpointer slot_ptr, MonoO
 }
 
 void
-mono_gc_wbarrier_arrayref_copy_internal (gpointer dest_ptr, gpointer src_ptr, int count)
+mono_gc_wbarrier_arrayref_copy_internal (gpointer dest_ptr, gconstpointer src_ptr, int count)
 {
 	mono_gc_memmove_aligned (dest_ptr, src_ptr, count * sizeof (gpointer));
 }
 
 void
-mono_gc_wbarrier_generic_store_internal (gpointer ptr, MonoObject* value)
+mono_gc_wbarrier_generic_store_internal (void volatile* ptr, MonoObject* value)
 {
 	*(void**)ptr = value;
 }
@@ -923,7 +923,7 @@ mono_gc_wbarrier_generic_nostore_internal (gpointer ptr)
 }
 
 void
-mono_gc_wbarrier_value_copy_internal (gpointer dest, gpointer src, int count, MonoClass *klass)
+mono_gc_wbarrier_value_copy_internal (gpointer dest, gconstpointer src, int count, MonoClass *klass)
 {
 	mono_gc_memmove_atomic (dest, src, count * mono_class_value_size (klass, NULL));
 }
@@ -959,371 +959,9 @@ mono_gc_get_restart_signal (void)
 }
 
 #if defined(USE_COMPILER_TLS) && defined(__linux__) && (defined(__i386__) || defined(__x86_64__))
-extern __thread void* GC_thread_tls;
-#include "metadata-internals.h"
-
-static int
-shift_amount (int v)
-{
-	int i = 0;
-	while (!(v & (1 << i)))
-		i++;
-	return i;
-}
-
-enum {
-	ATYPE_FREEPTR,
-	ATYPE_FREEPTR_FOR_BOX,
-	ATYPE_NORMAL,
-	ATYPE_GCJ,
-	ATYPE_STRING,
-	ATYPE_NUM
-};
-
-static MonoMethod*
-create_allocator (int atype, int tls_key, gboolean slowpath)
-{
-	int index_var, bytes_var, my_fl_var, my_entry_var;
-	guint32 no_freelist_branch, not_small_enough_branch = 0;
-	guint32 size_overflow_branch = 0;
-	MonoMethodBuilder *mb;
-	MonoMethod *res;
-	MonoMethodSignature *csig;
-	const char *name = NULL;
-	WrapperInfo *info;
-
-	g_assert_not_reached ();
-
-	if (atype == ATYPE_FREEPTR) {
-		name = slowpath ? "SlowAllocPtrfree" : "AllocPtrfree";
-	} else if (atype == ATYPE_FREEPTR_FOR_BOX) {
-		name = slowpath ? "SlowAllocPtrfreeBox" : "AllocPtrfreeBox";
-	} else if (atype == ATYPE_NORMAL) {
-		name = slowpath ? "SlowAlloc" : "Alloc";
-	} else if (atype == ATYPE_GCJ) {
-		name = slowpath ? "SlowAllocGcj" : "AllocGcj";
-	} else if (atype == ATYPE_STRING) {
-		name = slowpath ? "SlowAllocString" : "AllocString";
-	} else {
-		g_assert_not_reached ();
-	}
-
-	csig = mono_metadata_signature_alloc (mono_defaults.corlib, 2);
-
-	if (atype == ATYPE_STRING) {
-		csig->ret = m_class_get_byval_arg (mono_defaults.string_class);
-		csig->params [0] = mono_get_int_type ();
-		csig->params [1] = mono_get_int32_type ();
-	} else {
-		csig->ret = mono_get_object_type ();
-		csig->params [0] = mono_get_int_type ();
-		csig->params [1] = mono_get_int32_type ();
-	}
-
-	mb = mono_mb_new (mono_defaults.object_class, name, MONO_WRAPPER_ALLOC);
-
-	if (slowpath)
-		goto always_slowpath;
-
-	bytes_var = mono_mb_add_local (mb, mono_get_int32_type ());
-	if (atype == ATYPE_STRING) {
-		/* a string alloator method takes the args: (vtable, len) */
-		/* bytes = (offsetof (MonoString, chars) + ((len + 1) * 2)); */
-		mono_mb_emit_ldarg (mb, 1);
-		mono_mb_emit_icon (mb, 1);
-		mono_mb_emit_byte (mb, MONO_CEE_ADD);
-		mono_mb_emit_icon (mb, 1);
-		mono_mb_emit_byte (mb, MONO_CEE_SHL);
-		// sizeof (MonoString) might include padding
-		mono_mb_emit_icon (mb, G_STRUCT_OFFSET (MonoString, chars));
-		mono_mb_emit_byte (mb, MONO_CEE_ADD);
-		mono_mb_emit_stloc (mb, bytes_var);
-	} else {
-		mono_mb_emit_ldarg (mb, 1);
-		mono_mb_emit_stloc (mb, bytes_var);
-	}
-
-	/* this is needed for strings/arrays only as the other big types are never allocated with this method */
-	if (atype == ATYPE_STRING) {
-		/* check for size */
-		/* if (!SMALL_ENOUGH (bytes)) jump slow_path;*/
-		mono_mb_emit_ldloc (mb, bytes_var);
-		mono_mb_emit_icon (mb, (NFREELISTS-1) * GRANULARITY);
-		not_small_enough_branch = mono_mb_emit_short_branch (mb, MONO_CEE_BGT_UN_S);
-		/* check for overflow */
-		mono_mb_emit_ldloc (mb, bytes_var);
-		mono_mb_emit_icon (mb, sizeof (MonoString));
-		size_overflow_branch = mono_mb_emit_short_branch (mb, MONO_CEE_BLE_UN_S);
-	}
-
-	/* int index = INDEX_FROM_BYTES(bytes); */
-	index_var = mono_mb_add_local (mb, mono_get_int32_type ());
-	
-	mono_mb_emit_ldloc (mb, bytes_var);
-	mono_mb_emit_icon (mb, GRANULARITY - 1);
-	mono_mb_emit_byte (mb, MONO_CEE_ADD);
-	mono_mb_emit_icon (mb, shift_amount (GRANULARITY));
-	mono_mb_emit_byte (mb, MONO_CEE_SHR_UN);
-	mono_mb_emit_icon (mb, shift_amount (sizeof (gpointer)));
-	mono_mb_emit_byte (mb, MONO_CEE_SHL);
-	/* index var is already adjusted into bytes */
-	mono_mb_emit_stloc (mb, index_var);
-
-	my_fl_var = mono_mb_add_local (mb, mono_get_int_type ());
-	my_entry_var = mono_mb_add_local (mb, mono_get_int_type ());
-	/* my_fl = ((GC_thread)tsd) -> ptrfree_freelists + index; */
-	mono_mb_emit_byte (mb, MONO_CUSTOM_PREFIX);
-	mono_mb_emit_byte (mb, 0x0D); /* CEE_MONO_TLS */
-	mono_mb_emit_i4 (mb, tls_key);
-	if (atype == ATYPE_FREEPTR || atype == ATYPE_FREEPTR_FOR_BOX || atype == ATYPE_STRING)
-		mono_mb_emit_icon (mb, G_STRUCT_OFFSET (struct GC_Thread_Rep, tlfs)
-					+ G_STRUCT_OFFSET (struct thread_local_freelists,
-							   ptrfree_freelists));
-	else if (atype == ATYPE_NORMAL)
-		mono_mb_emit_icon (mb, G_STRUCT_OFFSET (struct GC_Thread_Rep, tlfs)
-					+ G_STRUCT_OFFSET (struct thread_local_freelists,
-							   normal_freelists));
-	else if (atype == ATYPE_GCJ)
-		mono_mb_emit_icon (mb, G_STRUCT_OFFSET (struct GC_Thread_Rep, tlfs)
-					+ G_STRUCT_OFFSET (struct thread_local_freelists,
-							   gcj_freelists));
-	else
-		g_assert_not_reached ();
-	mono_mb_emit_byte (mb, MONO_CEE_ADD);
-	mono_mb_emit_ldloc (mb, index_var);
-	mono_mb_emit_byte (mb, MONO_CEE_ADD);
-	mono_mb_emit_stloc (mb, my_fl_var);
-
-	/* my_entry = *my_fl; */
-	mono_mb_emit_ldloc (mb, my_fl_var);
-	mono_mb_emit_byte (mb, MONO_CEE_LDIND_I);
-	mono_mb_emit_stloc (mb, my_entry_var);
-
-	/* if (EXPECT((word)my_entry >= HBLKSIZE, 1)) { */
-	mono_mb_emit_ldloc (mb, my_entry_var);
-	mono_mb_emit_icon (mb, HBLKSIZE);
-	no_freelist_branch = mono_mb_emit_short_branch (mb, MONO_CEE_BLT_UN_S);
-
-	/* ptr_t next = obj_link(my_entry); *my_fl = next; */
-	mono_mb_emit_ldloc (mb, my_fl_var);
-	mono_mb_emit_ldloc (mb, my_entry_var);
-	mono_mb_emit_byte (mb, MONO_CEE_LDIND_I);
-	mono_mb_emit_byte (mb, MONO_CEE_STIND_I);
-
-	/* set the vtable and clear the words in the object */
-	mono_mb_emit_ldloc (mb, my_entry_var);
-	mono_mb_emit_ldarg (mb, 0);
-	mono_mb_emit_byte (mb, MONO_CEE_STIND_I);
-
-	if (atype == ATYPE_FREEPTR) {
-		int start_var, end_var, start_loop;
-		/* end = my_entry + bytes; start = my_entry + sizeof (gpointer);
-		 */
-		start_var = mono_mb_add_local (mb, mono_get_int_type ());
-		end_var = mono_mb_add_local (mb, mono_get_int_type ());
-		mono_mb_emit_ldloc (mb, my_entry_var);
-		mono_mb_emit_ldloc (mb, bytes_var);
-		mono_mb_emit_byte (mb, MONO_CEE_ADD);
-		mono_mb_emit_stloc (mb, end_var);
-		mono_mb_emit_ldloc (mb, my_entry_var);
-		mono_mb_emit_icon (mb, G_STRUCT_OFFSET (MonoObject, synchronisation));
-		mono_mb_emit_byte (mb, MONO_CEE_ADD);
-		mono_mb_emit_stloc (mb, start_var);
-		/*
-		 * do {
-		 * 	*start++ = NULL;
-		 * } while (start < end);
-		 */
-		start_loop = mono_mb_get_label (mb);
-		mono_mb_emit_ldloc (mb, start_var);
-		mono_mb_emit_icon (mb, 0);
-		mono_mb_emit_byte (mb, MONO_CEE_STIND_I);
-		mono_mb_emit_ldloc (mb, start_var);
-		mono_mb_emit_icon (mb, sizeof (gpointer));
-		mono_mb_emit_byte (mb, MONO_CEE_ADD);
-		mono_mb_emit_stloc (mb, start_var);
-
-		mono_mb_emit_ldloc (mb, start_var);
-		mono_mb_emit_ldloc (mb, end_var);
-		mono_mb_emit_byte (mb, MONO_CEE_BLT_UN_S);
-		mono_mb_emit_byte (mb, start_loop - (mono_mb_get_label (mb) + 1));
-	} else if (atype == ATYPE_FREEPTR_FOR_BOX || atype == ATYPE_STRING) {
-		/* need to clear just the sync pointer */
-		mono_mb_emit_ldloc (mb, my_entry_var);
-		mono_mb_emit_icon (mb, G_STRUCT_OFFSET (MonoObject, synchronisation));
-		mono_mb_emit_byte (mb, MONO_CEE_ADD);
-		mono_mb_emit_icon (mb, 0);
-		mono_mb_emit_byte (mb, MONO_CEE_STIND_I);
-	}
-
-	if (atype == ATYPE_STRING) {
-		/* need to set length and clear the last char */
-		/* s->length = len; */
-		mono_mb_emit_ldloc (mb, my_entry_var);
-		mono_mb_emit_icon (mb, G_STRUCT_OFFSET (MonoString, length));
-		mono_mb_emit_byte (mb, MONO_CEE_ADD);
-		mono_mb_emit_ldarg (mb, 1);
-		mono_mb_emit_byte (mb, MONO_CEE_STIND_I4);
-		/* s->chars [len] = 0; */
-		mono_mb_emit_ldloc (mb, my_entry_var);
-		mono_mb_emit_ldloc (mb, bytes_var);
-		mono_mb_emit_icon (mb, 2);
-		mono_mb_emit_byte (mb, MONO_CEE_SUB);
-		mono_mb_emit_byte (mb, MONO_CEE_ADD);
-		mono_mb_emit_icon (mb, 0);
-		mono_mb_emit_byte (mb, MONO_CEE_STIND_I2);
-	}
-
-	/* return my_entry; */
-	mono_mb_emit_ldloc (mb, my_entry_var);
-	mono_mb_emit_byte (mb, MONO_CEE_RET);
-	
-	mono_mb_patch_short_branch (mb, no_freelist_branch);
-	if (not_small_enough_branch > 0)
-		mono_mb_patch_short_branch (mb, not_small_enough_branch);
-	if (size_overflow_branch > 0)
-		mono_mb_patch_short_branch (mb, size_overflow_branch);
-
-	/* the slow path: we just call back into the runtime */
- always_slowpath:
-	if (atype == ATYPE_STRING) {
-		mono_mb_emit_ldarg (mb, 1);
-		mono_mb_emit_icall (mb, ves_icall_string_alloc);
-	} else {
-		mono_mb_emit_ldarg (mb, 0);
-		mono_mb_emit_icall (mb, ves_icall_object_new_specific);
-	}
-
-	mono_mb_emit_byte (mb, MONO_CEE_RET);
-
-	info = mono_wrapper_info_create (mb, WRAPPER_SUBTYPE_NONE);
-	info->d.alloc.gc_name = "boehm";
-	info->d.alloc.alloc_type = atype;
-	mb->init_locals = FALSE;
-
-	res = mono_mb_create (mb, csig, 8, info);
-	mono_mb_free (mb);
-
-	return res;
-}
-
-static MonoMethod* alloc_method_cache [ATYPE_NUM];
-static MonoMethod* slowpath_alloc_method_cache [ATYPE_NUM];
-
-gboolean
-mono_gc_is_critical_method (MonoMethod *method)
-{
-	int i;
-
-	for (i = 0; i < ATYPE_NUM; ++i)
-		if (method == alloc_method_cache [i] || method == slowpath_alloc_method_cache [i])
-			return TRUE;
-
-	return FALSE;
-}
-
-/*
- * If possible, generate a managed method that can quickly allocate objects in class
- * @klass. The method will typically have an thread-local inline allocation sequence.
- * The signature of the called method is:
- * 	object allocate (MonoVTable *vtable)
- * The thread local alloc logic is taken from libgc/pthread_support.c.
- */
-MonoMethod*
-mono_gc_get_managed_allocator (MonoClass *klass, gboolean for_box, gboolean known_instance_size)
-{
-	int atype;
-
-	/*
-	 * Tls implementation changed, we jump to tls native getters/setters.
-	 * Is boehm managed allocator ok with this ? Do we even care ?
-	 */
-	return NULL;
-
-	if (!SMALL_ENOUGH (m_class_get_instance_size (klass)))
-		return NULL;
-	if (mono_class_has_finalizer (klass) || mono_class_is_marshalbyref (klass))
-		return NULL;
-	if (G_UNLIKELY (mono_profiler_allocations_enabled ()))
-		return NULL;
-	if (m_class_get_rank (klass))
-		return NULL;
-	if (mono_class_is_open_constructed_type (m_class_get_byval_arg (klass)))
-		return NULL;
-	if (m_class_get_byval_arg (klass)->type == MONO_TYPE_STRING) {
-		atype = ATYPE_STRING;
-	} else if (!known_instance_size) {
-		return NULL;
-	} else if (!m_class_has_references (klass)) {
-		if (for_box)
-			atype = ATYPE_FREEPTR_FOR_BOX;
-		else
-			atype = ATYPE_FREEPTR;
-	} else {
-		return NULL;
-		/*
-		 * disabled because we currently do a runtime choice anyway, to
-		 * deal with multiple appdomains.
-		if (vtable->gc_descr != GC_NO_DESCRIPTOR)
-			atype = ATYPE_GCJ;
-		else
-			atype = ATYPE_NORMAL;
-		*/
-	}
-	return mono_gc_get_managed_allocator_by_type (atype, MANAGED_ALLOCATOR_REGULAR);
-}
-
-MonoMethod*
-mono_gc_get_managed_array_allocator (MonoClass *klass)
-{
-	return NULL;
-}
-
-/**
- * mono_gc_get_managed_allocator_by_type:
- *
- *   Return a managed allocator method corresponding to allocator type ATYPE.
- */
-MonoMethod*
-mono_gc_get_managed_allocator_by_type (int atype, ManagedAllocatorVariant variant)
-{
-	MonoMethod *res;
-	gboolean slowpath = variant != MANAGED_ALLOCATOR_REGULAR;
-	MonoMethod **cache = slowpath ? slowpath_alloc_method_cache : alloc_method_cache;
-
-	return NULL;
-
-	res = cache [atype];
-	if (res)
-		return res;
-
-	res = create_allocator (atype, -1, slowpath);
-	mono_os_mutex_lock (&mono_gc_lock);
-	if (cache [atype]) {
-		mono_free_method (res);
-		res = cache [atype];
-	} else {
-		mono_memory_barrier ();
-		cache [atype] = res;
-	}
-	mono_os_mutex_unlock (&mono_gc_lock);
-	return res;
-}
-
-guint32
-mono_gc_get_managed_allocator_types (void)
-{
-	return ATYPE_NUM;
-}
-
-MonoMethod*
-mono_gc_get_write_barrier (void)
-{
-	g_assert_not_reached ();
-	return NULL;
-}
-
-#else
+// Look at history around late August 2019 if this is to be restored.
+// The code was effectively dead, not merely deleted to avoid maintaining it.
+#endif
 
 gboolean
 mono_gc_is_critical_method (MonoMethod *method)
@@ -1362,8 +1000,6 @@ mono_gc_get_write_barrier (void)
 	return NULL;
 }
 
-#endif
-
 MonoMethod*
 mono_gc_get_specific_write_barrier (gboolean is_concurrent)
 {
@@ -1683,17 +1319,17 @@ mono_gc_register_finalizer_callbacks (MonoGCFinalizerCallbacks *callbacks)
 
 #define BITMAP_SIZE (sizeof (*((HandleData *)NULL)->bitmap) * CHAR_BIT)
 
-static inline gboolean
+static gboolean
 slot_occupied (HandleData *handles, guint slot) {
 	return handles->bitmap [slot / BITMAP_SIZE] & (1 << (slot % BITMAP_SIZE));
 }
 
-static inline void
+static void
 vacate_slot (HandleData *handles, guint slot) {
 	handles->bitmap [slot / BITMAP_SIZE] &= ~(1 << (slot % BITMAP_SIZE));
 }
 
-static inline void
+static void
 occupy_slot (HandleData *handles, guint slot) {
 	handles->bitmap [slot / BITMAP_SIZE] |= 1 << (slot % BITMAP_SIZE);
 }
@@ -2067,6 +1703,12 @@ mono_gchandle_free_domain (MonoDomain *domain)
 
 }
 
+guint64
+mono_gc_get_total_allocated_bytes (MonoBoolean precise) 
+{
+	return 0;
+}
+
 void
 mono_gc_register_obj_with_weak_fields (void *obj)
 {
