@@ -110,6 +110,24 @@ find_system_class (const char *name)
 		return NULL;
 }
 
+static void
+mono_custom_modifiers_get_desc (GString *res, const MonoType *type, gboolean include_namespace)
+{
+	ERROR_DECL (error);
+	int count = mono_type_custom_modifier_count (type);
+	for (int i = 0; i < count; ++i) {
+		gboolean required;
+		MonoType *cmod_type = mono_type_get_custom_modifier (type, i, &required, error);
+		mono_error_assert_ok (error);
+		if (required)
+			g_string_append (res, " modreq(");
+		else
+			g_string_append (res, " modopt(");
+		mono_type_get_desc (res, cmod_type, include_namespace);
+		g_string_append (res, ")");
+	}
+}
+
 void
 mono_type_get_desc (GString *res, MonoType *type, gboolean include_namespace)
 {
@@ -214,6 +232,9 @@ mono_type_get_desc (GString *res, MonoType *type, gboolean include_namespace)
 	default:
 		break;
 	}
+	if (type->has_cmods) {
+		mono_custom_modifiers_get_desc (res, type, include_namespace);
+	}
 	if (type->byref)
 		g_string_append_c (res, '&');
 }
