@@ -34,6 +34,7 @@
 #include <mono/utils/w32api.h>
 #include <mono/utils/mono-os-wait.h>
 #include "external-only.h"
+#include "icall-decl.h"
 
 /*
  * Pull the list of opcodes
@@ -90,19 +91,19 @@ static int array_size = 16;
 
 /* MonoThreadsSync status helpers */
 
-static inline guint32
+static guint32
 mon_status_get_owner (guint32 status)
 {
 	return status & OWNER_MASK;
 }
 
-static inline guint32
+static guint32
 mon_status_set_owner (guint32 status, guint32 owner)
 {
 	return (status & ENTRY_COUNT_MASK) | owner;
 }
 
-static inline gint32
+static gint32
 mon_status_get_entry_count (guint32 status)
 {
 	gint32 entry_count = (gint32)((status & ENTRY_COUNT_MASK) >> ENTRY_COUNT_SHIFT);
@@ -110,13 +111,13 @@ mon_status_get_entry_count (guint32 status)
 	return entry_count - zero;
 }
 
-static inline guint32
+static guint32
 mon_status_init_entry_count (guint32 status)
 {
 	return (status & OWNER_MASK) | ENTRY_COUNT_ZERO;
 }
 
-static inline guint32
+static guint32
 mon_status_add_entry_count (guint32 status, int val)
 {
 	if (val > 0)
@@ -125,7 +126,7 @@ mon_status_add_entry_count (guint32 status, int val)
 		return status - ((-val) << ENTRY_COUNT_SHIFT);
 }
 
-static inline gboolean
+static gboolean
 mon_status_have_waiters (guint32 status)
 {
 	return status & ENTRY_COUNT_WAITERS;
@@ -133,26 +134,26 @@ mon_status_have_waiters (guint32 status)
 
 /* LockWord helpers */
 
-static inline MonoThreadsSync*
+static MonoThreadsSync*
 lock_word_get_inflated_lock (LockWord lw)
 {
 	lw.lock_word &= (~LOCK_WORD_STATUS_MASK);
 	return lw.sync;
 }
 
-static inline gboolean
+static gboolean
 lock_word_is_inflated (LockWord lw)
 {
 	return lw.lock_word & LOCK_WORD_INFLATED;
 }
 
-static inline gboolean
+static gboolean
 lock_word_has_hash (LockWord lw)
 {
 	return lw.lock_word & LOCK_WORD_HAS_HASH;
 }
 
-static inline LockWord
+static LockWord
 lock_word_set_has_hash (LockWord lw)
 {
 	LockWord nlw;
@@ -160,26 +161,26 @@ lock_word_set_has_hash (LockWord lw)
 	return nlw;
 }
 
-static inline gboolean
+static gboolean
 lock_word_is_free (LockWord lw)
 {
 	return !lw.lock_word;
 }
 
-static inline gboolean
+static gboolean
 lock_word_is_flat (LockWord lw)
 {
 	/* Return whether the lock is flat or free */
 	return (lw.lock_word & LOCK_WORD_STATUS_MASK) == LOCK_WORD_FLAT;
 }
 
-static inline gint32
+static gint32
 lock_word_get_hash (LockWord lw)
 {
 	return (gint32) (lw.lock_word >> LOCK_WORD_HASH_SHIFT);
 }
 
-static inline gint32
+static gint32
 lock_word_get_nest (LockWord lw)
 {
 	if (lock_word_is_free (lw))
@@ -188,39 +189,39 @@ lock_word_get_nest (LockWord lw)
 	return ((lw.lock_word & LOCK_WORD_NEST_MASK) >> LOCK_WORD_NEST_SHIFT) + 1;
 }
 
-static inline gboolean
+static gboolean
 lock_word_is_nested (LockWord lw)
 {
 	return lw.lock_word & LOCK_WORD_NEST_MASK;
 }
 
-static inline gboolean
+static gboolean
 lock_word_is_max_nest (LockWord lw)
 {
 	return (lw.lock_word & LOCK_WORD_NEST_MASK) == LOCK_WORD_NEST_MASK;
 }
 
-static inline LockWord
+static LockWord
 lock_word_increment_nest (LockWord lw)
 {
 	lw.lock_word += 1 << LOCK_WORD_NEST_SHIFT;
 	return lw;
 }
 
-static inline LockWord
+static LockWord
 lock_word_decrement_nest (LockWord lw)
 {
 	lw.lock_word -= 1 << LOCK_WORD_NEST_SHIFT;
 	return lw;
 }
 
-static inline gint32
+static gint32
 lock_word_get_owner (LockWord lw)
 {
 	return lw.lock_word >> LOCK_WORD_OWNER_SHIFT;
 }
 
-static inline LockWord
+static LockWord
 lock_word_new_thin_hash (gint32 hash)
 {
 	LockWord lw;
@@ -229,7 +230,7 @@ lock_word_new_thin_hash (gint32 hash)
 	return lw;
 }
 
-static inline LockWord
+static LockWord
 lock_word_new_inflated (MonoThreadsSync *mon)
 {
 	LockWord lw;
@@ -238,7 +239,7 @@ lock_word_new_inflated (MonoThreadsSync *mon)
 	return lw;
 }
 
-static inline LockWord
+static LockWord
 lock_word_new_flat (gint32 owner)
 {
 	LockWord lw;
@@ -561,6 +562,7 @@ int
 mono_object_hash_internal (MonoObject* obj)
 {
 #ifdef HAVE_MOVING_COLLECTOR
+
 	LockWord lw;
 	unsigned int hash;
 	if (!obj)
@@ -619,13 +621,22 @@ mono_object_hash_internal (MonoObject* obj)
 	mono_memory_write_barrier ();
 	obj->synchronisation = lw.sync;
 	return hash;
+
 #else
+
 /*
  * Wang's address-based hash function:
  *   http://www.concentric.net/~Ttwang/tech/addrhash.htm
  */
 	return (GPOINTER_TO_UINT (obj) >> MONO_OBJECT_ALIGNMENT_SHIFT) * 2654435761u;
 #endif
+
+}
+
+int
+mono_object_hash_icall (MonoObjectHandle obj, MonoError* error)
+{
+	return mono_object_hash_internal (MONO_HANDLE_RAW (obj));
 }
 
 /*
@@ -637,6 +648,7 @@ mono_object_hash_internal (MonoObject* obj)
 int
 mono_object_hash (MonoObject* obj)
 {
+	// FIXME slow?
 	MONO_EXTERNAL_ONLY (int, mono_object_hash_internal (obj));
 }
 
@@ -782,7 +794,7 @@ signal_monitor (gpointer mon_untyped)
 /* If allow_interruption==TRUE, the method will be interrupted if abort or suspend
  * is requested. In this case it returns -1.
  */ 
-static inline gint32 
+static gint32
 mono_monitor_try_enter_inflated (MonoObject *obj, guint32 ms, gboolean allow_interruption, guint32 id)
 {
 	LockWord lw;
@@ -969,7 +981,7 @@ retry_contended:
  * If allow_interruption == TRUE, the method will be interrupted if abort or suspend
  * is requested. In this case it returns -1.
  */
-static inline gint32
+static gint32
 mono_monitor_try_enter_internal (MonoObject *obj, guint32 ms, gboolean allow_interruption)
 {
 	LockWord lw;
@@ -1062,6 +1074,7 @@ mono_monitor_enter_internal (MonoObject *obj)
 gboolean
 mono_monitor_enter (MonoObject *obj)
 {
+	// FIXME slow?
 	MONO_EXTERNAL_ONLY (gboolean, mono_monitor_enter_internal (obj));
 }
 
@@ -1117,6 +1130,12 @@ mono_monitor_exit_internal (MonoObject *obj)
 		mono_monitor_exit_flat (obj, lw);
 }
 
+void
+mono_monitor_exit_icall (MonoObjectHandle obj, MonoError* error)
+{
+	mono_monitor_exit_internal (MONO_HANDLE_RAW (obj));
+}
+
 /**
  * mono_monitor_exit:
  */
@@ -1159,24 +1178,38 @@ mono_monitor_threads_sync_members_offset (int *status_offset, int *nest_offset)
 	*nest_offset = ENCODE_OFF_SIZE (MONO_STRUCT_OFFSET (MonoThreadsSync, nest), sizeof (ts.nest));
 }
 
-void
-ves_icall_System_Threading_Monitor_Monitor_try_enter_with_atomic_var (MonoObject *obj, guint32 ms, MonoBoolean *lockTaken)
+static void
+mono_monitor_try_enter_with_atomic_var (MonoObject *obj, guint32 ms, MonoBoolean *lockTaken, MonoError* error)
 {
+	// The use of error here is unusual, but expedient, and easy enough to understand.
+	// Maybe clean it up later.
+
 	gint32 res;
 	gboolean allow_interruption = TRUE;
+
 	if (G_UNLIKELY (!obj)) {
-		ERROR_DECL (error);
-		mono_error_set_argument_null (error, "obj", "");
-		mono_error_set_pending_exception (error);
+		if (error) {
+			mono_error_set_argument_null (error, "obj", "");
+		} else {
+			ERROR_DECL (error);
+			mono_error_set_argument_null (error, "obj", "");
+			mono_error_set_pending_exception (error);
+		}
 		return;
 	}
+
 	do {
 		res = mono_monitor_try_enter_internal (obj, ms, allow_interruption);
 		/*This means we got interrupted during the wait and didn't got the monitor.*/
 		if (res == -1) {
 			MonoException *exc = mono_thread_interruption_checkpoint ();
 			if (exc) {
-				mono_set_pending_exception (exc);
+				if (error) { // implies icall and coop handle frame -- a little gross
+					MONO_HANDLE_NEW (MonoException, exc);
+					mono_error_set_exception_instance (error, exc);
+				} else {
+					mono_set_pending_exception (exc);
+				}
 				return;
 			} else {
 				//we detected a pending interruption but it turned out to be a false positive, we ignore it from now on (this feels like a hack, right?, threads.c should give us less confusing directions)
@@ -1184,10 +1217,17 @@ ves_icall_System_Threading_Monitor_Monitor_try_enter_with_atomic_var (MonoObject
 			}
 		}
 	} while (res == -1);
+
 	/*It's safe to do it from here since interruption would happen only on the wrapper.*/
 	*lockTaken = res == 1;
 }
 
+void
+ves_icall_System_Threading_Monitor_Monitor_try_enter_with_atomic_var (MonoObjectHandle obj, guint32 ms, MonoBoolean* lockTaken, MonoError* error)
+{
+	mono_monitor_try_enter_with_atomic_var (MONO_HANDLE_RAW (obj), ms, lockTaken, error);
+}
+
 /**
  * mono_monitor_enter_v4:
  */
@@ -1195,7 +1235,7 @@ void
 mono_monitor_enter_v4 (MonoObject *obj, char *lock_taken)
 {
 	g_static_assert (sizeof (MonoBoolean) == 1);
-	mono_monitor_enter_v4_internal  (obj, (MonoBoolean*)lock_taken);
+	mono_monitor_enter_v4_internal (obj, (MonoBoolean*)lock_taken);
 }
 
 /* Called from JITted code */
@@ -1208,8 +1248,7 @@ mono_monitor_enter_v4_internal (MonoObject *obj, MonoBoolean *lock_taken)
 		mono_error_set_pending_exception (error);
 		return;
 	}
-
-	ves_icall_System_Threading_Monitor_Monitor_try_enter_with_atomic_var (obj, MONO_INFINITE_WAIT, lock_taken);
+	mono_monitor_try_enter_with_atomic_var (obj, MONO_INFINITE_WAIT, lock_taken, NULL);
 }
 
 /*
@@ -1223,18 +1262,19 @@ mono_monitor_enter_v4_internal (MonoObject *obj, MonoBoolean *lock_taken)
 guint32
 mono_monitor_enter_v4_fast (MonoObject *obj, MonoBoolean *lock_taken)
 {
-	if (*lock_taken == 1)
+	if (*lock_taken == 1 || G_UNLIKELY (!obj))
 		return FALSE;
-	if (G_UNLIKELY (!obj))
-		return FALSE;
-	gint32 res = mono_monitor_try_enter_internal (obj, 0, TRUE);
-	*lock_taken = res == 1;
-	return res == 1;
+
+	gboolean const res = mono_monitor_try_enter_internal (obj, 0, TRUE) == 1;
+	*lock_taken = (MonoBoolean)res;
+	return (guint32)res;
 }
 
 MonoBoolean
-ves_icall_System_Threading_Monitor_Monitor_test_owner (MonoObject *obj)
+ves_icall_System_Threading_Monitor_Monitor_test_owner (MonoObjectHandle obj_handle, MonoError* error)
 {
+	MonoObject* const obj = MONO_HANDLE_RAW (obj_handle);
+
 	LockWord lw;
 
 	LOCK_DEBUG (g_message ("%s: Testing if %p is owned by thread %d", __func__, obj, mono_thread_info_get_small_id()));
@@ -1247,12 +1287,14 @@ ves_icall_System_Threading_Monitor_Monitor_test_owner (MonoObject *obj)
 		return mon_status_get_owner (lock_word_get_inflated_lock (lw)->status) == mono_thread_info_get_small_id ();
 	}
 	
-	return(FALSE);
+	return FALSE;
 }
 
 MonoBoolean
-ves_icall_System_Threading_Monitor_Monitor_test_synchronised (MonoObject *obj)
+ves_icall_System_Threading_Monitor_Monitor_test_synchronised (MonoObjectHandle obj_handle, MonoError* error)
 {
+	MonoObject* const obj = MONO_HANDLE_RAW (obj_handle);
+
 	LockWord lw;
 
 	LOCK_DEBUG (g_message("%s: (%d) Testing if %p is owned by any thread", __func__, mono_thread_info_get_small_id (), obj));
@@ -1307,20 +1349,22 @@ mono_monitor_pulse (MonoObject *obj, const char *func, gboolean all)
 }
 
 void
-ves_icall_System_Threading_Monitor_Monitor_pulse (MonoObject *obj)
+ves_icall_System_Threading_Monitor_Monitor_pulse (MonoObjectHandle obj, MonoError* error)
 {
-	mono_monitor_pulse (obj, __func__, FALSE);
+	mono_monitor_pulse (MONO_HANDLE_RAW (obj), __func__, FALSE);
 }
 
 void
-ves_icall_System_Threading_Monitor_Monitor_pulse_all (MonoObject *obj)
+ves_icall_System_Threading_Monitor_Monitor_pulse_all (MonoObjectHandle obj, MonoError* error)
 {
-	mono_monitor_pulse (obj, __func__, TRUE);
+	mono_monitor_pulse (MONO_HANDLE_RAW (obj), __func__, TRUE);
 }
 
 MonoBoolean
-ves_icall_System_Threading_Monitor_Monitor_wait (MonoObject *obj, guint32 ms)
+ves_icall_System_Threading_Monitor_Monitor_wait (MonoObjectHandle obj_handle, guint32 ms, MonoError* error)
 {
+	MonoObject* const obj = MONO_HANDLE_RAW (obj_handle);
+
 	LockWord lw;
 	MonoThreadsSync *mon;
 	HANDLE event;
@@ -1434,7 +1478,19 @@ ves_icall_System_Threading_Monitor_Monitor_wait (MonoObject *obj, guint32 ms)
 }
 
 void
-ves_icall_System_Threading_Monitor_Monitor_Enter (MonoObject *obj)
+ves_icall_System_Threading_Monitor_Monitor_Enter (MonoObjectHandle obj, MonoError* error)
 {
-	mono_monitor_enter_internal (obj);
+	mono_monitor_enter_internal (MONO_HANDLE_RAW (obj));
 }
+
+#if ENABLE_NETCORE
+gint64
+ves_icall_System_Threading_Monitor_Monitor_LockContentionCount (void)
+{
+#ifndef DISABLE_PERFCOUNTERS
+	return mono_perfcounters->thread_contentions;
+#else
+	return 0;
+#endif
+}
+#endif
\ No newline at end of file
