@@ -637,13 +637,13 @@ mono_config_for_assembly_internal (MonoImage *assembly)
 	}
 
 	for (i = 0; (aname = get_assembly_filename (assembly, i)) != NULL; ++i) {
-		cfg = g_build_filename (cfg_dir, "mono", "assemblies", aname, cfg_name, NULL);
+		cfg = g_build_filename (cfg_dir, "mono", "assemblies", aname, cfg_name, (const char*)NULL);
 		got_it += mono_config_parse_file_with_context (&state, cfg);
 		g_free (cfg);
 
 #ifdef TARGET_WIN32
 		const char *home = g_get_home_dir ();
-		cfg = g_build_filename (home, ".mono", "assemblies", aname, cfg_name, NULL);
+		cfg = g_build_filename (home, ".mono", "assemblies", aname, cfg_name, (const char*)NULL);
 		got_it += mono_config_parse_file_with_context (&state, cfg);
 		g_free (cfg);
 #endif
@@ -676,14 +676,14 @@ mono_config_parse (const char *filename)
 
 	const char *cfg_dir = mono_get_config_dir ();
 	if (cfg_dir) {
-		char *mono_cfg = g_build_filename (cfg_dir, "mono", "config", (const char *)NULL);
+		char *mono_cfg = g_build_filename (cfg_dir, "mono", "config", (const char*)NULL);
 		mono_config_parse_file (mono_cfg);
 		g_free (mono_cfg);
 	}
 
 #if !defined(TARGET_WIN32)
 	home = g_get_home_dir ();
-	char *user_cfg = g_strconcat (home, G_DIR_SEPARATOR_S, ".mono/config", (const char *)NULL);
+	char *user_cfg = g_strconcat (home, G_DIR_SEPARATOR_S, ".mono/config", (const char*)NULL);
 	mono_config_parse_file (user_cfg);
 	g_free (user_cfg);
 #endif
