@@ -152,14 +152,14 @@ ves_icall_System_Security_Principal_WindowsIdentity_GetUserToken (MonoStringHand
 }
 
 #if G_HAVE_API_SUPPORT(HAVE_CLASSIC_WINAPI_SUPPORT)
-MonoArray*
-ves_icall_System_Security_Principal_WindowsIdentity_GetRoles (gpointer token)
+MonoArrayHandle
+ves_icall_System_Security_Principal_WindowsIdentity_GetRoles (gpointer token, MonoError *error)
 {
-	ERROR_DECL (error);
-	MonoArray *array = NULL;
+	MonoArrayHandle array;
+	MonoStringHandle str_h;
 	MonoDomain *domain = mono_domain_get ();
-
 	gint32 size = 0;
+	gboolean created = FALSE;
 
 	GetTokenInformation (token, TokenGroups, NULL, size, (PDWORD)&size);
 	if (size > 0) {
@@ -168,25 +168,27 @@ ves_icall_System_Security_Principal_WindowsIdentity_GetRoles (gpointer token)
 			int i=0;
 			int num = tg->GroupCount;
 
-			array = mono_array_new_checked (domain, mono_get_string_class (), num, error);
-			if (mono_error_set_pending_exception (error)) {
+			array = mono_array_new_handle (domain, mono_get_string_class (), num, error);
+			if (!is_ok (error)) {
 				g_free (tg);
-				return NULL;
+				return NULL_HANDLE_ARRAY;
 			}
+			created = TRUE;
 
+			str_h = MONO_HANDLE_NEW (MonoString, NULL);
 			for (i=0; i < num; i++) {
 				gint32 size = 0;
 				gunichar2 *uniname = GetSidName (NULL, tg->Groups [i].Sid, &size);
 
 				if (uniname) {
 					MonoString *str = mono_string_new_utf16_checked (domain, uniname, size, error);
+					MONO_HANDLE_ASSIGN_RAW (str_h, str);
 					if (!is_ok (error)) {
 						g_free (uniname);
 						g_free (tg);
-						mono_error_set_pending_exception (error);
-						return NULL;
+						return NULL_HANDLE_ARRAY;
 					}
-					mono_array_setref_internal (array, i, str);
+					MONO_HANDLE_ARRAY_SETREF (array, i, str_h);
 					g_free (uniname);
 				}
 			}
@@ -194,10 +196,10 @@ ves_icall_System_Security_Principal_WindowsIdentity_GetRoles (gpointer token)
 		g_free (tg);
 	}
 
-	if (!array) {
+	if (!created) {
 		/* return empty array of string, i.e. string [0] */
-		array = mono_array_new_checked (domain, mono_get_string_class (), 0, error);
-		mono_error_set_pending_exception (error);
+		array = mono_array_new_handle (domain, mono_get_string_class (), 0, error);
+		return_val_if_nok (error, NULL_HANDLE_ARRAY);
 	}
 	return array;
 }
