@@ -22,6 +22,9 @@
 #include <mono/utils/strenc.h>
 #include "reflection-internals.h"
 #include "icall-decl.h"
+
+#ifndef ENABLE_NETCORE
+
 #ifndef HOST_WIN32
 #ifdef HAVE_GRP_H
 #include <grp.h>
@@ -39,19 +42,23 @@
 
 #if defined(__GNUC__)
 
+#ifdef HAVE_GRP_H
 #ifndef HAVE_GETGRGID_R
 	#warning Non-thread safe getgrgid being used!
 #endif
 #ifndef HAVE_GETGRNAM_R
 	#warning Non-thread safe getgrnam being used!
 #endif
+#endif
+
+#ifdef HAVE_PWD_H
 #ifndef HAVE_GETPWNAM_R
 	#warning Non-thread safe getpwnam being used!
 #endif
 #ifndef HAVE_GETPWUID_R
 	#warning Non-thread safe getpwuid being used!
 #endif
-
+#endif
 #endif /* defined(__GNUC__) */
 #endif /* !HOST_WIN32 */
 
@@ -607,11 +614,12 @@ mono_invoke_protected_memory_method (MonoArrayHandle data, MonoObjectHandle scop
 {
 	if (!*method) {
 		MonoDomain *domain = mono_domain_get ();
+		MonoAssemblyLoadContext *alc = mono_domain_default_alc (domain);
 		if (system_security_assembly == NULL) {
-			system_security_assembly = mono_image_loaded_internal (mono_domain_default_alc (domain), "System.Security", FALSE);
+			system_security_assembly = mono_image_loaded_internal (alc, "System.Security", FALSE);
 			if (!system_security_assembly) {
 				MonoAssemblyOpenRequest req;
-				mono_assembly_request_prepare (&req.request, sizeof (req), MONO_ASMCTX_DEFAULT);
+				mono_assembly_request_prepare_open (&req, MONO_ASMCTX_DEFAULT, alc);
 				MonoAssembly *sa = mono_assembly_request_open ("System.Security.dll", &req, NULL);
 				g_assert (sa);
 				system_security_assembly = mono_assembly_get_image_internal (sa);
@@ -640,3 +648,9 @@ ves_icall_System_Security_SecureString_EncryptInternal (MonoArrayHandle data, Mo
 {
 	mono_invoke_protected_memory_method (data, scope, "Protect", &mono_method_securestring_encrypt, error);
 }
+
+#else
+
+MONO_EMPTY_SOURCE_FILE (mono_security);
+
+#endif /* ENABLE_NETCORE */
