@@ -21,6 +21,8 @@
 #include "utils/mono-rand.h"
 #include "icall-decl.h"
 
+#ifndef ENABLE_NETCORE
+
 MonoBoolean
 ves_icall_System_Security_Cryptography_RNGCryptoServiceProvider_RngOpen (MonoError *error)
 {
@@ -46,3 +48,9 @@ ves_icall_System_Security_Cryptography_RNGCryptoServiceProvider_RngClose (gpoint
 {
 	mono_rand_close (handle);
 }
+
+#else
+
+MONO_EMPTY_SOURCE_FILE (rand);
+
+#endif /* ENABLE_NETCORE */
