@@ -63,6 +63,12 @@ mono_w32file_create(const gunichar2 *name, guint32 fileaccess, guint32 sharemode
 	return res;
 }
 
+gboolean
+mono_w32file_cancel (gpointer handle)
+{
+	return CancelIoEx (handle, NULL);
+}
+
 gboolean
 mono_w32file_close (gpointer handle)
 {
