@@ -25,7 +25,10 @@ gpointer
 ves_icall_System_Threading_Semaphore_CreateSemaphore_icall (gint32 initialCount, gint32 maximumCount,
 	const gunichar2 *name, gint32 name_length, gint32 *win32error, MonoError *error)
 { 
-	HANDLE sem = CreateSemaphoreW (NULL, initialCount, maximumCount, name);
+	HANDLE sem;
+	MONO_ENTER_GC_SAFE;
+	sem = CreateSemaphoreW (NULL, initialCount, maximumCount, name);
+	MONO_EXIT_GC_SAFE;
 	*win32error = GetLastError ();
 	return sem;
 }
@@ -41,7 +44,10 @@ gpointer
 ves_icall_System_Threading_Semaphore_OpenSemaphore_icall (const gunichar2 *name, gint32 name_length,
 	gint32 rights, gint32 *win32error, MonoError *error)
 {
-	HANDLE sem = OpenSemaphoreW (rights, FALSE, name);
+	HANDLE sem;
+	MONO_ENTER_GC_SAFE;
+	sem = OpenSemaphoreW (rights, FALSE, name);
+	MONO_EXIT_GC_SAFE;
 	*win32error = GetLastError ();
 	return sem;
 }
