@@ -10,28 +10,60 @@
 #include <stdio.h>
 #include "mintops.h"
 
-#define OPDEF(a,b,c,d) \
-	b,
-const char *mono_interp_opname[] = {
+// This, instead of an array of pointers, to optimize away a pointer and a relocation per string.
+struct MonoInterpOpnameCharacters {
+#define OPDEF(a,b,c,d,e,f) char a [sizeof (b)];
 #include "mintops.def"
-	""
 };
 #undef OPDEF
 
-#define OPDEF(a,b,c,d) \
-	c,
-unsigned char mono_interp_oplen[] = {
+const MonoInterpOpnameCharacters mono_interp_opname_characters = {
+#define OPDEF(a,b,c,d,e,f) b,
 #include "mintops.def"
-	0
 };
 #undef OPDEF
 
+const guint16 mono_interp_opname_offsets [] = {
+#define OPDEF(a,b,c,d,e,f) offsetof (MonoInterpOpnameCharacters, a),
+#include "mintops.def"
+#undef OPDEF
+};
 
-#define OPDEF(a,b,c,d) \
-	d,
-MintOpArgType mono_interp_opargtype[] = {
+#define OPDEF(a,b,c,d,e,f) c,
+unsigned char const mono_interp_oplen [] = {
+#include "mintops.def"
+};
+#undef OPDEF
+
+#define Push0 0
+#define Push1 1
+#define Push2 2
+#define Pop0 0
+#define Pop1 1
+#define Pop2 2
+#define Pop3 3
+#define Pop4 4
+#define Pop5 5
+#define Pop6 6
+#define PopAll MINT_POP_ALL
+#define VarPush MINT_VAR_PUSH
+#define VarPop MINT_VAR_POP
+
+#define OPDEF(a,b,c,d,e,f) d,
+int const mono_interp_oppop[] = {
+#include "mintops.def"
+};
+#undef OPDEF
+
+#define OPDEF(a,b,c,d,e,f) e,
+int const mono_interp_oppush[] = {
+#include "mintops.def"
+};
+#undef OPDEF
+
+#define OPDEF(a,b,c,d,e,f) f,
+MintOpArgType const mono_interp_opargtype [] = {
 #include "mintops.def"
-	(MintOpArgType)0
 };
 #undef OPDEF
 
@@ -58,7 +90,7 @@ mono_interp_dis_mintop(const guint16 *base, const guint16 *ip)
 	guint32 token;
 	int target;
 
-	g_string_append_printf (str, "IL_%04x: %-10s", (int)(ip - base), mono_interp_opname [*ip]);
+	g_string_append_printf (str, "IR_%04x: %-10s", (int)(ip - base), mono_interp_opname (*ip));
 	switch (mono_interp_opargtype [*ip]) {
 	case MintOpNoArgs:
 		break;
@@ -98,11 +130,11 @@ mono_interp_dis_mintop(const guint16 *base, const guint16 *ip)
 	}
 	case MintOpShortBranch:
 		target = ip + * (short *)(ip + 1) - base;
-		g_string_append_printf (str, " IL_%04x", target);
+		g_string_append_printf (str, " IR_%04x", target);
 		break;
 	case MintOpBranch:
 		target = ip + (gint32)READ32 (ip + 1) - base;
-		g_string_append_printf (str, " IL_%04x", target);
+		g_string_append_printf (str, " IR_%04x", target);
 		break;
 	case MintOpSwitch: {
 		const guint16 *p = ip + 1;
@@ -115,7 +147,7 @@ mono_interp_dis_mintop(const guint16 *base, const guint16 *ip)
 			if (i > 0)
 				g_string_append_printf (str, ", ");
 			offset = (gint32)READ32 (p);
-			g_string_append_printf (str, "IL_%04x", (int)(p + offset - base));
+			g_string_append_printf (str, "IR_%04x", (int)(p + offset - base));
 			p += 2;
 		}
 		g_string_append_printf (str, ")");
@@ -128,3 +160,8 @@ mono_interp_dis_mintop(const guint16 *base, const guint16 *ip)
 	return g_string_free (str, FALSE);
 }
 
+const char*
+mono_interp_opname (int op)
+{
+	return ((const char*)&mono_interp_opname_characters) + mono_interp_opname_offsets [op];
+}
