@@ -80,16 +80,22 @@ G_EXTERN_C gint64 mono_fconv_i8 (double v);
 G_EXTERN_C guint32 mono_fconv_u4 (double v);
 G_EXTERN_C guint32 mono_fconv_u4_2 (double v);
 
+G_EXTERN_C guint32 mono_rconv_u4 (float v);
+
 G_EXTERN_C gint64 mono_fconv_ovf_i8 (double v);
 
 G_EXTERN_C guint64 mono_fconv_ovf_u8 (double v);
 
+G_EXTERN_C guint64 mono_fconv_ovf_u8_un (double v);
+
 G_EXTERN_C gint64 mono_rconv_i8 (float v);
 
 G_EXTERN_C gint64 mono_rconv_ovf_i8 (float v);
 
 G_EXTERN_C guint64 mono_rconv_ovf_u8 (float v);
 
+G_EXTERN_C guint64 mono_rconv_ovf_u8_un (float v);
+
 G_EXTERN_C double mono_lconv_to_r8 (gint64 a);
 
 G_EXTERN_C double mono_conv_to_r8 (gint32 a);
