@@ -6,9 +6,6 @@
 //
 // Licensed under the MIT license. See LICENSE file in the project root for full license information.
 //
-// Mono's internal header files are not C++ clean, so avoid including them if 
-// possible
-//
 
 #include "config.h"
 
@@ -16,6 +13,7 @@
 #include <llvm-c/ExecutionEngine.h>
 
 #include "mini-llvm-cpp.h"
+#include "mini-runtime.h"
 #include "llvm-jit.h"
 
 #if defined(MONO_ARCH_LLVM_JIT_SUPPORTED) && !defined(MONO_CROSS_COMPILE) && LLVM_API_VERSION > 600
@@ -24,6 +22,7 @@
 #include <llvm/Support/Host.h>
 #include <llvm/Support/TargetSelect.h>
 #include <llvm/IR/Mangler.h>
+#include "llvm/IR/LegacyPassNameParser.h"
 #include <llvm/ExecutionEngine/ExecutionEngine.h>
 #include "llvm/ExecutionEngine/Orc/CompileUtils.h"
 #include "llvm/ExecutionEngine/Orc/IRCompileLayer.h"
@@ -31,6 +30,7 @@
 #include "llvm/ExecutionEngine/RTDyldMemoryManager.h"
 #include "llvm/ExecutionEngine/Orc/RTDyldObjectLinkingLayer.h"
 #include "llvm/ExecutionEngine/JITSymbol.h"
+#include "llvm/Transforms/Scalar.h"
 
 #include <cstdlib>
 
@@ -92,9 +92,20 @@ MonoJitMemoryManager::allocateDataSection(uintptr_t Size,
 										  unsigned Alignment,
 										  unsigned SectionID,
 										  StringRef SectionName,
-										  bool IsReadOnly) {
-	uint8_t *res = (uint8_t*)malloc (Size);
+										  bool IsReadOnly)
+{
+	uint8_t *res;
+
+	// FIXME: Use a mempool
+	if (Alignment == 32) {
+		/* Used for SIMD */
+		res = (uint8_t*)malloc (Size + 32);
+		res += (GPOINTER_TO_UINT (res) % 32);
+	} else {
+		res = (uint8_t*)malloc (Size);
+	}
 	assert (res);
+	g_assert (GPOINTER_TO_UINT (res) % Alignment == 0);
 	memset (res, 0, Size);
 	return res;
 }
@@ -176,7 +187,11 @@ struct MonoLLVMJIT {
 		auto resolver = createLegacyLookupResolver (execution_session,
 			lookup_name, on_error);
 		resolvers[k] = std::move (resolver);
-		compile_layer.addModule (k, std::move(m));
+		auto err = compile_layer.addModule (k, std::move(m));
+		if (err) {
+			outs () << "addModule error: " << err << "\n";
+			assert (0);
+		}
 		return k;
 	}
 
@@ -240,6 +255,12 @@ make_mono_llvm_jit (TargetMachine *target_machine)
 
 #elif LLVM_API_VERSION > 600
 
+// The OptimizationList is automatically populated with registered Passes by the
+// PassNameParser.
+//
+static cl::list<const PassInfo*, bool, PassNameParser>
+PassList(cl::desc("Optimizations available:"));
+
 class MonoLLVMJIT {
 public:
 	/* We use our own trampoline infrastructure instead of the Orc one */
@@ -250,7 +271,39 @@ public:
 	MonoLLVMJIT (TargetMachine *TM, MonoJitMemoryManager *mm)
 		: TM(TM), ObjectLayer([=] { return std::shared_ptr<RuntimeDyld::MemoryManager> (mm); }),
 		  CompileLayer (ObjectLayer, SimpleCompiler (*TM)),
-		  modules() {
+		  modules(),
+		  fpm (NULL) {
+		initPassManager ();
+	}
+
+	void initPassManager () {
+		PassRegistry &registry = *PassRegistry::getPassRegistry();
+		initializeCore(registry);
+		initializeScalarOpts(registry);
+		initializeInstCombine(registry);
+		initializeTarget(registry);
+
+		const char *opts = g_getenv ("MONO_LLVM_OPT");
+		if (opts == NULL) {
+			// FIXME: find optimal mono specific order of passes
+			// see https://llvm.org/docs/Frontend/PerformanceTips.html#pass-ordering
+			// the following order is based on a stripped version of "OPT -O2"
+			opts = " -simplifycfg -sroa -lower-expect -instcombine -licm -simplifycfg -lcssa -indvars -loop-deletion -gvn -memcpyopt -sccp -bdce -instcombine -dse -simplifycfg";
+		}
+
+		char **args = g_strsplit (opts, " ", -1);
+		llvm::cl::ParseCommandLineOptions (g_strv_length (args), args, "");
+
+		for (int i = 0; i < PassList.size(); i++) {
+			Pass *pass = PassList[i]->getNormalCtor()();
+			if (pass->getPassKind () == llvm::PT_Function || pass->getPassKind () == llvm::PT_Loop) {
+				fpm.add (pass);
+			} else {
+				printf("Opt pass is ignored: %s\n", args[i + 1]);
+			}
+		}
+		g_strfreev (args);
+		fpm.doInitialization();
 	}
 
 	ModuleHandleT addModule(Function *F, std::shared_ptr<Module> M) {
@@ -310,6 +363,7 @@ public:
 
 	gpointer compile (Function *F, int nvars, LLVMValueRef *callee_vars, gpointer *callee_addrs, gpointer *eh_frame) {
 		F->getParent ()->setDataLayout (TM->createDataLayout ());
+		fpm.run(*F);
 		// Orc uses a shared_ptr to refer to modules so we have to save them ourselves to keep a ref
 		std::shared_ptr<Module> m (F->getParent ());
 		modules.push_back (m);
@@ -339,6 +393,7 @@ private:
 	ObjLayerT ObjectLayer;
 	CompileLayerT CompileLayer;
 	std::vector<std::shared_ptr<Module>> modules;
+	legacy::FunctionPassManager fpm;
 };
 
 static MonoJitMemoryManager *mono_mm;
@@ -360,7 +415,7 @@ make_mono_llvm_jit (TargetMachine *target_machine)
 static MonoLLVMJIT *jit;
 
 MonoEERef
-mono_llvm_create_ee (LLVMModuleProviderRef MP, AllocCodeMemoryCb *alloc_cb, FunctionEmittedCb *emitted_cb, ExceptionTableCb *exception_cb, LLVMExecutionEngineRef *ee)
+mono_llvm_create_ee (AllocCodeMemoryCb *alloc_cb, FunctionEmittedCb *emitted_cb, ExceptionTableCb *exception_cb, LLVMExecutionEngineRef *ee)
 {
 	alloc_code_mem_cb = alloc_cb;
 
@@ -369,15 +424,21 @@ mono_llvm_create_ee (LLVMModuleProviderRef MP, AllocCodeMemoryCb *alloc_cb, Func
 
 	EnableMonoEH = true;
 	MonoEHFrameSymbol = "mono_eh_frame";
-
 	EngineBuilder EB;
-#if defined(TARGET_AMD64) || defined(TARGET_X86)
-	std::vector<std::string> attrs;
-	// FIXME: Autodetect this
-	attrs.push_back("sse3");
-	attrs.push_back("sse4.1");
-	EB.setMAttrs (attrs);
-#endif
+
+	if (mono_use_fast_math) {
+		TargetOptions opts;
+		opts.NoInfsFPMath = true;
+		opts.NoNaNsFPMath = true;
+		opts.NoSignedZerosFPMath = true;
+		opts.NoTrappingFPMath = true;
+		opts.UnsafeFPMath = true;
+		opts.AllowFPOpFusion = FPOpFusion::Fast;
+		EB.setTargetOptions (opts);
+	}
+
+	EB.setOptLevel(CodeGenOpt::Aggressive);
+	EB.setMCPU(sys::getHostCPUName());
 	auto TM = EB.selectTarget ();
 	assert (TM);
 
@@ -412,7 +473,7 @@ mono_llvm_set_unhandled_exception_handler (void)
 }
 
 MonoEERef
-mono_llvm_create_ee (LLVMModuleProviderRef MP, AllocCodeMemoryCb *alloc_cb, FunctionEmittedCb *emitted_cb, ExceptionTableCb *exception_cb, LLVMExecutionEngineRef *ee)
+mono_llvm_create_ee (AllocCodeMemoryCb *alloc_cb, FunctionEmittedCb *emitted_cb, ExceptionTableCb *exception_cb, LLVMExecutionEngineRef *ee)
 {
 	g_error ("LLVM JIT not supported on this platform.");
 	return NULL;
