@@ -1136,10 +1136,10 @@ mono_arm_get_thumb_plt_entry (guint8 *code)
 	i1 = (s ^ j1) ? 0 : 1;
 	i2 = (s ^ j2) ? 0 : 1;
 
-	imm32 = (imm11 << 1) | (imm10 << 12) | (i2 << 22) | (i1 << 23);
+	imm32 = (imm11 << 1) | (imm10 << 12) | (i2 << 22) | (i1 << 23) | (s << 24);
 	if (s)
-		/* Sign extend from 24 bits to 32 bits */
-		imm32 = ((gint32)imm32 << 8) >> 8;
+		/* Sign extend from 25 bits to 32 bits */
+		imm32 = ((gint32)imm32 << 7) >> 7;
 
 	target = code + imm32;
 
