@@ -269,6 +269,12 @@ public class Tests
 	[DllImport("libtest")]
 	public static extern int mono_test_cominterop_ccw_queryinterface ([MarshalAs (UnmanagedType.Interface)] IOtherTest itest);
 
+	[DllImport("libtest")]
+	public static extern int mono_test_cominterop_ccw_queryinterface_foreign_thread ([MarshalAs (UnmanagedType.Interface)] ITest itest);
+
+	[DllImport ("libtest")]
+	public static extern int mono_test_cominterop_ccw_itest_foreign_thread ([MarshalAs (UnmanagedType.Interface)] ITest itest);
+
 	[DllImport("libtest")]
 	public static extern int mono_test_marshal_safearray_out_1dim_vt_bstr_empty ([MarshalAs (UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_VARIANT)]out Array array);
 
@@ -321,6 +327,12 @@ public class Tests
 		[In, MarshalAs (UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_VARIANT)] Array array3,
 		[In, Out, MarshalAs (UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_VARIANT)] ref Array array4);
 
+	[DllImport("libtest")]
+	public static extern int mono_test_marshal_safearray_in_ccw([MarshalAs (UnmanagedType.Interface)] ITest itest);
+
+	[DllImport("libtest")]
+	public static extern int mono_test_default_interface_ccw([MarshalAs (UnmanagedType.Interface)] ITest itest);
+
 	[DllImport("libtest")]
 	public static extern bool mono_cominterop_is_supported ();
 
@@ -592,6 +604,7 @@ public class Tests
 			if (mono_test_cominterop_ccw_queryinterface (otherTest) != 0)
 				return 202;
 
+
 			if (mono_test_marshal_retval_ccw_itest(test, true) != 0)
 				return 203;
 
@@ -599,6 +612,22 @@ public class Tests
 			if (mono_test_marshal_retval_ccw_itest(test_pres_sig, false) != 0)
 				return 204;
 
+			if (mono_test_default_interface_ccw(test) != 0)
+				return 205;
+
+			if (mono_test_cominterop_ccw_queryinterface_foreign_thread (test) != 0)
+				return 206;
+
+			{
+				ManagedTest mt = new ManagedTest ();
+				if (mono_test_cominterop_ccw_itest_foreign_thread (test) != 0)
+					return 207;
+				if (mt.Status != 0) {
+					Console.Error.WriteLine ("after mono_test_cominterop_ccw_itest_foreign_thread Status = {0}", mt.Status);
+					return 208;
+				}
+			}
+
 			#endregion // COM Callable Wrapper Tests
 
 			#region SAFEARRAY tests
@@ -606,7 +635,6 @@ public class Tests
 			if (isWindows) {
 
 				/* out */
-
 				Array array;
 				if ((mono_test_marshal_safearray_out_1dim_vt_bstr_empty (out array) != 0) || (array.Rank != 1) || (array.Length != 0))
 					return 62;
@@ -762,6 +790,8 @@ public class Tests
 					if (i != Convert.ToInt32 (array4.GetValue (i)))
 						return 96;
 				}
+				if (mono_test_marshal_safearray_in_ccw(test) != 0)
+					return 97;
 			}
 			#endregion // SafeArray Tests
 
@@ -776,6 +806,56 @@ public class Tests
         return 0;
 	}
 
+	[ComImport ()]
+	[Guid ("10000000-0000-0000-0000-000000000002")]
+	[InterfaceType (ComInterfaceType.InterfaceIsIUnknown)]
+	public interface IDefTest1
+	{
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		int Method1();
+	}
+
+	[ComImport ()]
+	[Guid ("10000000-0000-0000-0000-000000000003")]
+	[InterfaceType (ComInterfaceType.InterfaceIsIUnknown)]
+	public interface IDefTest2
+	{
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		int Method2();
+	}
+
+	public class TestDefaultInterfaceClass : IDefTest1, IDefTest2
+	{
+		public int Method3()
+		{
+			return 3;
+		}
+
+		public int Method1()
+		{
+			return 1;
+		}
+
+		public int Method4()
+		{
+			return 4;
+		}
+
+		public int Method2()
+		{
+			return 2;
+		}
+	}
+
+	[ComDefaultInterfaceAttribute (typeof (IDefTest1))]
+	public class TestDefaultInterfaceClass1 : TestDefaultInterfaceClass
+	{
+	}
+
+	[ComDefaultInterfaceAttribute (typeof (IDefTest2))]
+	public class TestDefaultInterfaceClass2 : TestDefaultInterfaceClass
+	{
+	}
 
 	[ComImport ()]
 	[Guid ("00000000-0000-0000-0000-000000000001")]
@@ -817,6 +897,18 @@ public class Tests
 		int Return22NoICall();
 		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
 		int IntOut();
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		void ArrayIn ([In, MarshalAs (UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_VARIANT)] object[] array);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		void ArrayIn2 ([In] object[] array);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		void ArrayIn3 (object[] array);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		[return: MarshalAs (UnmanagedType.Interface)]
+		TestDefaultInterfaceClass1 GetDefInterface1();
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		[return: MarshalAs (UnmanagedType.Interface)]
+		TestDefaultInterfaceClass2 GetDefInterface2();
 	}
 
 	[ComImport ()]
@@ -873,6 +965,12 @@ public class Tests
 		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
 		[PreserveSig ()]
 		int IntOut (out int val);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		int ArrayIn ([In, MarshalAs (UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_VARIANT)] object[] array);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		int ArrayIn2 ([In] object[] array);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		int ArrayIn3 (object[] array);
 	}
 
 	[System.Runtime.InteropServices.GuidAttribute ("00000000-0000-0000-0000-000000000002")]
@@ -916,6 +1014,16 @@ public class Tests
 		public virtual extern int Return22NoICall();
 		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
 		public virtual extern int IntOut();
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		public virtual extern void ArrayIn ([In, MarshalAs (UnmanagedType.SafeArray, SafeArraySubType = VarEnum.VT_VARIANT)] object[] array);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		public virtual extern void ArrayIn2 ([In] object[] array);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		public virtual extern void ArrayIn3 (object[] array);
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		public virtual extern TestDefaultInterfaceClass1 GetDefInterface1();
+		[MethodImplAttribute (MethodImplOptions.InternalCall, MethodCodeType = MethodCodeType.Runtime)]
+		public virtual extern TestDefaultInterfaceClass2 GetDefInterface2();
 	}
 
 	[System.Runtime.InteropServices.GuidAttribute ("00000000-0000-0000-0000-000000000002")]
@@ -1066,6 +1174,44 @@ public class Tests
 			val = 33;
 			return 0;
 		}
+
+		public int ArrayIn(object[] array)
+		{
+			if (array.Length != 2)
+				return 40;
+			if (array.Rank != 1)
+				return 41;
+			if (array.GetLowerBound(0) != 0)
+				return 42;
+			if (array.GetUpperBound(0) != 1)
+				return 43;
+			if (array[0] is string)
+			{
+				if ((array[0] as string) != "Test")
+					return 44;
+			}
+			else
+				return 45;
+			if (array[1] is int)
+			{
+				if ((int)array[1] != 2345)
+					return 46;
+			}
+			else
+				return 47;
+
+			return 444;
+		}
+
+		public int ArrayIn2(object[] array)
+		{
+			return ArrayIn(array);
+		}
+
+		public int ArrayIn3(object[] array)
+		{
+			return ArrayIn(array);
+		}
 	}
 
 	public class ManagedTest : ITest
@@ -1165,6 +1311,50 @@ public class Tests
 		{
 			return 33;
 		}
+
+		public void ArrayIn(object[] array)
+		{
+			if (array.Length != 2)
+				status = 40;
+			else if (array.Rank != 1)
+				status = 41;
+			else if (array.GetLowerBound(0) != 0)
+				status = 42;
+			else if (array.GetUpperBound(0) != 1)
+				status = 43;
+			else if (array[0] is string)
+			{
+				if ((array[0] as string) != "Test")
+					status = 44;
+			}
+			else if (array[1] is int)
+			{
+				if ((int)array[1] != 2345)
+					status = 45;
+			}
+
+			status = 444;
+		}
+
+		public void ArrayIn2(object[] array)
+		{
+			ArrayIn(array);
+		}
+
+		public void ArrayIn3(object[] array)
+		{
+			ArrayIn(array);
+		}
+
+		public TestDefaultInterfaceClass1 GetDefInterface1()
+		{
+			return new TestDefaultInterfaceClass1();
+		}
+
+		public TestDefaultInterfaceClass2 GetDefInterface2()
+		{
+			return new TestDefaultInterfaceClass2();
+		}
 	}
 
 	[ComVisible (true)]
