@@ -12,13 +12,15 @@
 #include <assert.h>
 #include <glib.h>
 
-/* For dlmalloc.h */
-#define USE_DL_PREFIX 1
-
 #include "mono-codeman.h"
 #include "mono-mmap.h"
 #include "mono-counters.h"
+#if _WIN32
+static void* mono_code_manager_heap;
+#else
+#define USE_DL_PREFIX 1
 #include "dlmalloc.h"
+#endif
 #include <mono/metadata/profiler-private.h>
 #ifdef HAVE_VALGRIND_MEMCHECK_H
 #include <valgrind/memcheck.h>
@@ -39,11 +41,15 @@ static const MonoCodeManagerCallbacks *code_manager_callbacks;
  * malloc when using dynamic code managers. The system malloc can't do this so we use a 
  * slighly modified version of Doug Lea's Malloc package for this purpose:
  * http://g.oswego.edu/dl/html/malloc.html
+ *
+ * Or on Windows, HeapCreate (HEAP_CREATE_ENABLE_EXECUTE).
  */
 
 #define MIN_PAGES 16
 
-#if defined(__x86_64__) || defined (_WIN64)
+#if _WIN32 // These are the same.
+#define MIN_ALIGN MEMORY_ALLOCATION_ALIGNMENT
+#elif defined(__x86_64__)
 /*
  * We require 16 byte alignment on amd64 so the fp literals embedded in the code are 
  * properly aligned for SSE2.
@@ -65,29 +71,29 @@ static const MonoCodeManagerCallbacks *code_manager_callbacks;
 
 #define MONO_PROT_RWX (MONO_MMAP_READ|MONO_MMAP_WRITE|MONO_MMAP_EXEC|MONO_MMAP_JIT)
 
-typedef struct _CodeChunck CodeChunk;
+typedef struct _CodeChunk CodeChunk;
 
 enum {
 	CODE_FLAG_MMAP,
 	CODE_FLAG_MALLOC
 };
 
-struct _CodeChunck {
+struct _CodeChunk {
 	char *data;
+	CodeChunk *next;
 	int pos;
 	int size;
-	CodeChunk *next;
-	unsigned int flags: 8;
+	unsigned int reserved: 8;
 	/* this number of bytes is available to resolve addresses far in memory */
 	unsigned int bsize: 24;
 };
 
 struct _MonoCodeManager {
-	int dynamic;
-	int read_only;
 	CodeChunk *current;
 	CodeChunk *full;
 	CodeChunk *last;
+	int dynamic : 1;
+	int read_only : 1;
 };
 
 #define ALIGN_INT(val,alignment) (((val) + (alignment - 1)) & ~(alignment - 1))
@@ -184,6 +190,47 @@ mono_code_manager_install_callbacks (const MonoCodeManagerCallbacks* callbacks)
 	code_manager_callbacks = callbacks;
 }
 
+static
+int
+mono_codeman_allocation_type (MonoCodeManager const *cman)
+{
+#ifdef FORCE_MALLOC
+	return CODE_FLAG_MALLOC;
+#else
+	return cman->dynamic ? CODE_FLAG_MALLOC : CODE_FLAG_MMAP;
+#endif
+}
+
+/**
+ * mono_code_manager_new_internal
+ *
+ * Returns: the new code manager
+ */
+static
+MonoCodeManager*
+mono_code_manager_new_internal (gboolean dynamic)
+{
+	MonoCodeManager* cman = g_new0 (MonoCodeManager, 1);
+	if (cman) {
+		cman->dynamic = dynamic;
+#if _WIN32
+		// It would seem the heap should live and die with the codemanager,
+		// but that was failing, so try a global.
+		if (mono_codeman_allocation_type (cman) == CODE_FLAG_MALLOC && !mono_code_manager_heap) {
+			// This heap is leaked, similar to dlmalloc state.
+			void* const heap = HeapCreate (HEAP_CREATE_ENABLE_EXECUTE, 0, 0);
+			if (heap && mono_atomic_cas_ptr (&mono_code_manager_heap, heap, NULL))
+				HeapDestroy (heap);
+			if (!mono_code_manager_heap) {
+				mono_code_manager_destroy (cman);
+				cman = 0;
+			}
+		}
+#endif
+	}
+	return cman;
+}
+
 /**
  * mono_code_manager_new:
  *
@@ -198,7 +245,7 @@ mono_code_manager_install_callbacks (const MonoCodeManagerCallbacks* callbacks)
 MonoCodeManager* 
 mono_code_manager_new (void)
 {
-	return (MonoCodeManager *) g_malloc0 (sizeof (MonoCodeManager));
+	return mono_code_manager_new_internal (FALSE);
 }
 
 /**
@@ -213,14 +260,37 @@ mono_code_manager_new (void)
 MonoCodeManager* 
 mono_code_manager_new_dynamic (void)
 {
-	MonoCodeManager *cman = mono_code_manager_new ();
-	cman->dynamic = 1;
-	return cman;
+	return mono_code_manager_new_internal (TRUE);
 }
 
+static gpointer
+mono_codeman_malloc (gsize n)
+{
+#if _WIN32
+	void* const heap = mono_code_manager_heap;
+	g_assert (heap);
+	return HeapAlloc (heap, 0, n);
+#else
+	return dlmemalign (MIN_ALIGN, n);
+#endif
+}
 
 static void
-free_chunklist (CodeChunk *chunk)
+mono_codeman_free (gpointer p)
+{
+	if (!p)
+		return;
+#if _WIN32
+	void* const heap = mono_code_manager_heap;
+	g_assert (heap);
+	HeapFree (heap, 0, p);
+#else
+	dlfree (p);
+#endif
+}
+
+static void
+free_chunklist (MonoCodeManager *cman, CodeChunk *chunk)
 {
 	CodeChunk *dead;
 	
@@ -233,17 +303,22 @@ free_chunklist (CodeChunk *chunk)
 #define valgrind_unregister(x)
 #endif
 
+	if (!chunk)
+		return;
+
+	const int flags = mono_codeman_allocation_type (cman);
+
 	for (; chunk; ) {
 		dead = chunk;
 		MONO_PROFILER_RAISE (jit_chunk_destroyed, ((mono_byte *) dead->data));
 		if (code_manager_callbacks)
 			code_manager_callbacks->chunk_destroy (dead->data);
 		chunk = chunk->next;
-		if (dead->flags == CODE_FLAG_MMAP) {
+		if (flags == CODE_FLAG_MMAP) {
 			codechunk_vfree (dead->data, dead->size);
 			/* valgrind_unregister(dead->data); */
-		} else if (dead->flags == CODE_FLAG_MALLOC) {
-			dlfree (dead->data);
+		} else if (flags == CODE_FLAG_MALLOC) {
+			mono_codeman_free (dead->data);
 		}
 		code_memory_used -= dead->size;
 		g_free (dead);
@@ -258,8 +333,10 @@ free_chunklist (CodeChunk *chunk)
 void
 mono_code_manager_destroy (MonoCodeManager *cman)
 {
-	free_chunklist (cman->full);
-	free_chunklist (cman->current);
+	if (!cman)
+		return;
+	free_chunklist (cman, cman->full);
+	free_chunklist (cman, cman->current);
 	g_free (cman);
 }
 
@@ -270,7 +347,7 @@ mono_code_manager_destroy (MonoCodeManager *cman)
  * so that any attempt to execute code allocated in the code
  * manager \p cman will fail. This is used for debugging purposes.
  */
-void             
+void
 mono_code_manager_invalidate (MonoCodeManager *cman)
 {
 	CodeChunk *chunk;
@@ -292,7 +369,7 @@ mono_code_manager_invalidate (MonoCodeManager *cman)
  * \param cman a code manager
  * Make the code manager read only, so further allocation requests cause an assert.
  */
-void             
+void
 mono_code_manager_set_read_only (MonoCodeManager *cman)
 {
 	cman->read_only = TRUE;
@@ -320,7 +397,7 @@ mono_code_manager_foreach (MonoCodeManager *cman, MonoCodeManagerFunc func, void
 	}
 }
 
-/* BIND_ROOM is the divisor for the chunck of code size dedicated
+/* BIND_ROOM is the divisor for the chunk of code size dedicated
  * to binding branches (branches not reachable with the immediate displacement)
  * bind_size = size/BIND_ROOM;
  * we should reduce it and make MIN_PAGES bigger for such systems
@@ -333,26 +410,22 @@ mono_code_manager_foreach (MonoCodeManager *cman, MonoCodeManagerFunc func, void
 #endif
 
 static CodeChunk*
-new_codechunk (CodeChunk *last, int dynamic, int size)
+new_codechunk (MonoCodeManager *cman, int size)
 {
-	int minsize, flags = CODE_FLAG_MMAP;
+	CodeChunk * const last = cman->last;
+	int const dynamic = cman->dynamic;
 	int chunk_size, bsize = 0;
-	int pagesize, valloc_granule;
 	CodeChunk *chunk;
 	void *ptr;
 
-#ifdef FORCE_MALLOC
-	flags = CODE_FLAG_MALLOC;
-#endif
-
-	pagesize = mono_pagesize ();
-	valloc_granule = mono_valloc_granule ();
+	const int flags = mono_codeman_allocation_type (cman);
+	const int pagesize = mono_pagesize ();
+	const int valloc_granule = mono_valloc_granule ();
 
 	if (dynamic) {
 		chunk_size = size;
-		flags = CODE_FLAG_MALLOC;
 	} else {
-		minsize = MAX (pagesize * MIN_PAGES, valloc_granule);
+		const int minsize = MAX (pagesize * MIN_PAGES, valloc_granule);
 		if (size < minsize)
 			chunk_size = minsize;
 		else {
@@ -386,7 +459,7 @@ new_codechunk (CodeChunk *last, int dynamic, int size)
 #endif
 
 	if (flags == CODE_FLAG_MALLOC) {
-		ptr = dlmemalign (MIN_ALIGN, chunk_size + MIN_ALIGN - 1);
+		ptr = mono_codeman_malloc (chunk_size + MIN_ALIGN - 1);
 		if (!ptr)
 			return NULL;
 	} else {
@@ -400,17 +473,17 @@ new_codechunk (CodeChunk *last, int dynamic, int size)
 			return NULL;
 	}
 
-	if (flags == CODE_FLAG_MALLOC) {
 #ifdef BIND_ROOM
+	if (flags == CODE_FLAG_MALLOC) {
 		/* Make sure the thunks area is zeroed */
 		memset (ptr, 0, bsize);
-#endif
 	}
+#endif
 
 	chunk = (CodeChunk *) g_malloc (sizeof (CodeChunk));
 	if (!chunk) {
 		if (flags == CODE_FLAG_MALLOC)
-			dlfree (ptr);
+			mono_codeman_free (ptr);
 		else
 			mono_vfree (ptr, chunk_size, MONO_MEM_ACCOUNT_CODE);
 		return NULL;
@@ -418,7 +491,7 @@ new_codechunk (CodeChunk *last, int dynamic, int size)
 	chunk->next = NULL;
 	chunk->size = chunk_size;
 	chunk->data = (char *) ptr;
-	chunk->flags = flags;
+	chunk->reserved = 0;
 	chunk->pos = bsize;
 	chunk->bsize = bsize;
 	if (code_manager_callbacks)
@@ -459,7 +532,7 @@ mono_code_manager_reserve_align (MonoCodeManager *cman, int size, int alignment)
 	}
 
 	if (!cman->current) {
-		cman->current = new_codechunk (cman->last, cman->dynamic, size);
+		cman->current = new_codechunk (cman, size);
 		if (!cman->current)
 			return NULL;
 		cman->last = cman->current;
@@ -492,7 +565,7 @@ mono_code_manager_reserve_align (MonoCodeManager *cman, int size, int alignment)
 		cman->full = chunk;
 		break;
 	}
-	chunk = new_codechunk (cman->last, cman->dynamic, size);
+	chunk = new_codechunk (cman, size);
 	if (!chunk)
 		return NULL;
 	chunk->next = cman->current;
