@@ -367,6 +367,13 @@ mono_vfree (void *addr, size_t length, MonoMemAccountType type)
  */
 void*
 mono_file_map (size_t length, int flags, int fd, guint64 offset, void **ret_handle)
+{
+	return mono_file_map_error (length, flags, fd, offset, ret_handle, NULL, NULL);
+}
+
+void*
+mono_file_map_error (size_t length, int flags, int fd, guint64 offset, void **ret_handle,
+	const char *filepath, char **error_message)
 {
 	void *ptr;
 	int mflags = 0;
@@ -386,25 +393,26 @@ mono_file_map (size_t length, int flags, int fd, guint64 offset, void **ret_hand
 #ifdef HOST_WASM
 	if (length == 0)
 		/* emscripten throws an exception on 0 length */
+		*error_message = g_stdrup_printf ("%s failed file:%s length:0x%zx offset:0x%lluX error:%s\n",
+			__func__, filepath ? filepath : "", length, offset, "mmaps of zero length are not permitted with emscripten");
 		return NULL;
 #endif
 
+	// No GC safe transition because this is called early in main.c
 	BEGIN_CRITICAL_SECTION;
 	ptr = mmap (0, length, prot, mflags, fd, offset);
 	END_CRITICAL_SECTION;
-	if (ptr == MAP_FAILED)
+	if (ptr == MAP_FAILED) {
+		if (error_message) {
+			*error_message = g_strdup_printf ("%s failed file:%s length:0x%zX offset:0x%lluX error:%s(0x%X)\n",
+				__func__, filepath ? filepath : "", length, (unsigned long long)offset, g_strerror (errno), errno);
+		}
 		return NULL;
+	}
 	*ret_handle = (void*)length;
 	return ptr;
 }
 
-void*
-mono_file_map_error (size_t length, int flags, int fd, guint64 offset, void **ret_handle,
-	const char *filepath, char **error_message)
-{
-	return mono_file_map (length, flags, fd, offset, ret_handle);
-}
-
 /**
  * mono_file_unmap:
  * \param addr memory address returned by mono_file_map ()
@@ -418,6 +426,7 @@ mono_file_unmap (void *addr, void *handle)
 {
 	int res;
 
+	// No GC safe transition because this is called early in driver.c via mono_debug_init (with a few layers of indirection)
 	BEGIN_CRITICAL_SECTION;
 	res = munmap (addr, (size_t)handle);
 	END_CRITICAL_SECTION;
@@ -459,6 +468,7 @@ mono_mprotect (void *addr, size_t length, int flags)
 #endif
 #endif
 	}
+	// No GC safe transition because this is called early in mini_init via mono_arch_init (with a few layers of indirection)
 	return mprotect (addr, length, prot);
 }
 
